﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class Brush
    {
        internal int _handle = 0;

        public bool equals(Brush value)
        {
            return value._handle == _handle;
        }

        /// <summary>
        /// 
        /// </summary>
        public float Alpha
        {
            set
            {
                Xors3d.BrushAlpha(_handle, value);
            }
            get
            {
                return Xors3d.GetBrushAlpha(_handle);
            }
        }

        /// <summary>
        /// free brush handle
        /// </summary>
        public void Free()
        {
            Xors3d.FreeBrush(_handle);
        }

        /// <summary>
        /// Returns texture assigned to brush
        /// </summary>
        /// <param name="layer">Texture layer index. Must be in range 0-7</param>
        /// <returns></returns>
        public Texture GetTexture(int layer=0)
        {
            Texture tex = new Texture();
            tex._handle = Xors3d.GetBrushTexture(_handle, layer);
            return tex._handle > 0 ? tex : null;
        }

        /// <summary>
        /// The green, red and blue values should be in the range [0; 255]. The default brush color is 255, 255, 255. Please note that if xEntityFX() or xBrushFX() flag VERTEXCOLOR is being used, brush colour will have no effect and vertex colours will be used instead.
        /// </summary>
        public Color Color
        {
            set
            {
                Xors3d.BrushColor(_handle, value._red, value._green, value._blue);
            }
            get
            {
                Color c = new Color();
                c._red = Xors3d.GetBrushRed(_handle);
                c._green = Xors3d.GetBrushGreen(_handle);
                c._blue = Xors3d.GetBrushBlue(_handle);
                return c;
            }
        }

        /// <summary>
        /// Sets or gets the specular shininess of a brush. 
        /// The shininess value should be in the range [0.0; 1.0]. The default shininess setting is 0.0 Shininess is how much brighter certain areas of an object will appear to be when a light is shone directly at them. Setting a shininess value of 1.0 for a medium to high poly sphere, combined with the creation of a light shining in the direction of it, will give it the appearance of a shiny snooker ball 
        /// </summary>
        public float Shininess
        {
            set
            {
                Xors3d.BrushShininess(_handle, value);
            }
            get
            {
                return Xors3d.GetBrushShininess(_handle);
            }
        }

        /// <summary>
        /// Sets or gets the blending mode for a brush.
        /// </summary>
        public Blend Blend
        {
            get
            {
                return (Blend)Xors3d.GetBrushBlend(_handle);
            }
            set
            {
                Xors3d.BrushBlend(_handle, (int)value);
            }
        }

        /// <summary>
        /// Sets miscellaneous effects for a brush. 
        /// Flags can be added to combine two or more effects
        /// </summary>
        public FX FX
        {
            get
            {
                return (FX)Xors3d.GetBrushFX(_handle);
            }
            set
            {
                Xors3d.BrushFX(_handle, (int)value);
            }
        }

        /// <summary>
        /// Assigns a texture to a brush. 
        /// The optional frame parameter specifies which animation frame, if any exist, should be assigned to the brush. The optional index parameter specifies texture layer that the texture should be assigned to.
        /// </summary>
        /// <param name="texture"></param>
        /// <param name="frame"></param>
        /// <param name="index"></param>
        public void SetTexture(Texture texture, int frame = 0, int index = 0)
        {
            Xors3d.BrushTexture(_handle, texture._handle, frame, index);
        }

        /// <summary>
        /// Sets or gets brush's name
        /// </summary>
        public string Name
        {
            set
            {
                Xors3d.BrushName(_handle, value);
            }
            get
            {
                return Xors3d.GetBrushName(_handle);
            }
        }

    }
}
