﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class Effect
    {
        internal int _handle = 0;

        public bool equals(Effect value)
        {
            return value._handle == _handle;
        }

        /// <summary>
        /// Frees up an effect
        /// </summary>
        public void Free()
        {
            Xors3d.FreeEffect(_handle);
        }

        /// <summary>
        /// Assings shader effect to entity
        /// </summary>
        /// <param name="entity"></param>
        /// <param name="index">Index of the effect layer in range [0; 7]. If -1 is passed - active layer will be used </param>
        public void SetEntity(Entity entity, int index=-1)
        {
            Xors3d.SetEntityEffect(entity._handle, _handle, index);
        }

        /// <summary>
        /// Assings shader effect to surface
        /// </summary>
        /// <param name="entity"></param>
        /// <param name="index">Index of the effect layer in range [0; 7]. If -1 is passed - active layer will be used </param>
        public void SetSurface(Surface surface, int index = -1)
        {
            Xors3d.SetSurfaceEffect(surface._handle, _handle, index);
        }

        /// <summary>
        /// Returns true if hardware supports specified technique
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public bool ValidateTechnique(string name)
        {
            return Xors3d.ValidateEffectTechnique(_handle, name) == 1;
        }


        public void SetEffectInt(string param, int value)
        {
            Xors3d.SetFXInt(_handle, param, value);
        }

        public void SetEffectBool(string param, bool value)
        {
            Xors3d.SetFXBool(_handle, param, value);
        }

        public void SetEffectFloat(string param, float value)
        {
            Xors3d.SetFXFloat(_handle, param, value);
        }

        public void SetEffectVector(string param, Vector3 value, float w = 0)
        {
            Xors3d.SetFXVector(_handle, param, value._x, value._y, value._z, w);
        }

        public void SetEffectTexture(string param, Texture value,int frame=0)
        {
            Xors3d.SetFXTexture(_handle, param, value._handle,frame);
        }

        public void SetEffectVectorArray(string param, IntPtr value, int count)
        {
            Xors3d.SetFXVectorArray(_handle, param, value.ToInt32(), count);
        }

        public void SetEffectMatrixArray(string param, IntPtr value, int count)
        {
            Xors3d.SetFXMatrixArray(_handle, param, value.ToInt32(), count);
        }

        public void SetEffectFloatArray(string param, IntPtr value, int count)
        {
            Xors3d.SetFXFloatArray(_handle, param, value.ToInt32(), count);
        }

        public void SetEffectIntArray(string param, IntPtr value, int count)
        {
            Xors3d.SetFXIntArray(_handle, param, value.ToInt32(), count);
        }

        

        public void SetEffectTextureOfEntity(string param, int index)
        {
            Xors3d.SetFXTexture(_handle, param, index);
        }

        public void DeleteEffectConstant(string name)
        {
            Xors3d.DeleteFXConstant(_handle, name);
        }

        public void ClearEffectConstants()
        {
            Xors3d.ClearFXConstants(_handle);
        }

        public void xSetEffectTechnique(string name)
        {
            Xors3d.SetFXTechnique(_handle, name);
        }
    }
}
