﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class File
    {
        internal int _handle = 0;

        public bool equals(File value)
        {
            return value._handle == _handle;
        }

        /// <summary>
        /// Closes opened file handle
        /// </summary>
        public void Close()
        {
            Xors3d.CloseFile(_handle);
        }

        /// <summary>
        /// Returns the current position within a file that is being processed following xReadFile(), xWriteFile() or xOpenFile()
        /// </summary>
        /// <returns></returns>
        public int Pos()
        {
            return Xors3d.FilePos(_handle);
        }

        /// <summary>
        /// Sets current position in file
        /// </summary>
        /// <param name="offset">Offset in bytes from the start of file </param>
        public void Seek(int offset)
        {
            Xors3d.SeekFile(_handle, offset);
        }

        /// <summary>
        /// Checks to see if the End of File of an opened file or stream has been reached
        /// </summary>
        /// <returns></returns>
        public bool isEof()
        {
            return Xors3d.Eof(_handle) == 1;
        }

        public int ReadShort()
        {
            return Xors3d.ReadShort(_handle);
        }

        public int ReadInt()
        {
            return Xors3d.ReadInt(_handle);
        }

        public int ReadByte()
        {
            return Xors3d.ReadByte(_handle);
        }

        public float ReadFloat()
        {
            return Xors3d.ReadFloat(_handle);
        }

        public string ReadString()
        {
            return Xors3d.ReadString(_handle);
        }

        /// <summary>
        /// Reads a line from file. Line should be terminated with a line separator specified with ls_flag parameter
        /// </summary>
        /// <param name="separ"></param>
        /// <returns></returns>
        public string ReadLine(TLineSepar separ= TLineSepar.LS_NUL)
        {
            return Xors3d.ReadLine(_handle,(int)separ);
        }

        public void WriteInt(int value)
        {
            Xors3d.WriteInt(_handle, value);
        }
        public void WriteShort(int value)
        {
            Xors3d.WriteShort(_handle, value);
        }
        public void WriteByte(int value)
        {
            Xors3d.WriteByte(_handle, value);
        }
        public void WriteFloat(float value)
        {
            Xors3d.WriteFloat(_handle, value);
        }
        public void WriteString(string value)
        {
            Xors3d.WriteString(_handle, value);
        }
        public void WriteLine(string value, TLineSepar separ = TLineSepar.LS_NUL)
        {
            Xors3d.WriteLine(_handle, value, (int)separ);
        }

    }

    public enum TLineSepar
    {
        LS_CR = 1,
        LS_CRLF = 3,
        LS_LF = 2,
        LS_NUL = 0
    }
}
