﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class Fog
    {
        internal Camera _camera = null;

        /// <summary>
        /// Sets the camera fog mode. 
        /// This will enable/disable fogging, a technique used to gradually fade out graphics the further they are away from the camera. This can be used to avoid 'pop-up', the moment at which 3D objects suddenly appear on the horizon. The default fog colour is black and the default fog range is 1-1000, although these can be changed by using xCameraFogColor() and xCameraFogRange() respectively. Each camera can have its own fog mode, for multiple on-screen fog effects.
        /// </summary>
        /// <param name="type"></param>
        public void SetFogMode(FogType type)
        {
            Xors3d.CameraFogMode(_camera._handle, (int)type);
        }

        /// <summary>
        /// Sets camera fog color. Defaults to 0, 0, 0
        /// </summary>
        /// <param name="color"></param>
        public void SetFogColor(Color color)
        {
            Xors3d.CameraFogColor(_camera._handle, color._red, color._green, color._blue);
        }

        /// <summary>
        /// Sets camera fog range. 
        /// The 'nearRange' parameter specifies at what distance in front of the camera that the fogging effect will start; all 3D object before this point will not be faded. The 'farRange' parameter specifies at what distance in front of the camera that the fogging effect will end; all 3D objects beyond this point will be completely faded out. 
        /// </summary>
        /// <param name="near">Distance in front of camera that fog starts </param>
        /// <param name="range">Distance in front of camera that fog ends </param>
        public void SetFogRange(float near, float range)
        {
            Xors3d.CameraFogRange(_camera._handle, near, range);
        }
    }
}
