﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public static class Graphics
    {

        static internal bool _Bloom = false;
        static internal PostEffectPoly _Bloom_poly = null;
        static internal Texture texture = null;
        static internal Texture texture2 = null;
        /// <summary>
        /// Activate bloom of shader
        /// </summary>
        public static void SetBloom(Camera camera, string shaderPath)
        {
            _Bloom_poly = ShaderEffects.CreatePostEffectPoly(camera, 1);
            texture = XE.CreateTexture(256, 256);
            texture2 = XE.CreateTexture(GetGraphicsWidth(), GetGraphicsHeight());

            // load posteffect shader
            Effect shader =  ShaderEffects.LoadFXFile(shaderPath);
            if (shader != null)
            {
                _Bloom_poly.ShaderEffect.SetEffect(shader);
                _Bloom_poly.ShaderEffect.SetEffectTechnique("Diffuse");
                _Bloom_poly.ShaderEffect.SetEffectMatrixSemantic("MatWorldViewProj", EffectMatrixSemantic.WORLDVIEWPROJ);
                _Bloom_poly.ShaderEffect.SetEffectTexture("tDiffuse", texture);
                _Bloom_poly.ShaderEffect.SetEffectTexture("tEmissive", texture2);
                _Bloom = true;
            }
        }

        internal static void DrawBloom()
        {
            if (_Bloom == true)
            {
                StretchBackBuffer(texture, 0, 0, 256, 256, (TextureFilter)0);
                StretchBackBuffer(texture2, 0, 0, GetGraphicsWidth(), GetGraphicsHeight(), (TextureFilter)0);
                _Bloom_poly.ShaderEffect.SetEffectTechnique("Diffuse");
                _Bloom_poly.RenderPostEffect();
                StretchBackBuffer(texture, 0, 0, 256, 256, (TextureFilter)0);
                _Bloom_poly.ShaderEffect.SetEffectTechnique("DiffuseH");
                _Bloom_poly.RenderPostEffect();
                StretchBackBuffer(texture, 0, 0, 256, 256, (TextureFilter)0);
                _Bloom_poly.ShaderEffect.SetEffectTechnique("DiffuseV");
                _Bloom_poly.RenderPostEffect();
            }
        }


        internal static void UpdateBumps()
        {
            foreach (Entity ent in XE.bumps)
            {
                ent.ShaderEffect.SetEffectVector("cameraPosition", ent._bump_camera.Movement.GetPosition());
            }
        }

        /// <summary>
        /// Animates all entities in the world, and performs collision checking. The speed parameter allows you affect the animation speed of all entities at once. A value of 1.0 will animate entities at their usual animation speed, a value of 2.0 will animate entities at double their animation speed, and so on. For best results use this command once per main loop, just before calling xRenderWorld(). 
        /// </summary>
        public static void UpdateWorld(float speed = 1)
        {
            Xors3d.UpdateWorld(speed);
            UpdateBumps();
        }

        /// <summary>
        /// Returns true if specified windows message occurs. 
        /// Parameters:
        /// message Windows message to be checked. "WM_CLOSE" is the only proper message presently 
        /// </summary>
        /// <param name="message"></param>
        public static bool WinMessage(string message)
        {
            return Xors3d.WinMessage(message) == 1;
        }

        /// <summary>
        /// Draws a line, in the current drawing color
        /// </summary>
        /// <param name="x1">start</param>
        /// <param name="y1"></param>
        /// <param name="x2">end</param>
        /// <param name="y2"></param>
        public static void DrawLine(int x1, int y1, int x2, int y2)
        {
            Xors3d.Line(x1, y1, x2, y2);
        }

        /// <summary>
        /// Draws a rectangle in the current drawing color starting at the location specified
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="solid"></param>
        public static void DrawRect(int x, int y, int width, int height, bool solid = false)
        {
            Xors3d.Rect(x, y, width, height, solid);
        }

        /// <summary>
        /// Checks if two rectangles are overlapping
        /// </summary>
        /// <param name="x1"></param>
        /// <param name="y1"></param>
        /// <param name="width1"></param>
        /// <param name="height1"></param>
        /// <param name="x2"></param>
        /// <param name="y2"></param>
        /// <param name="width2"></param>
        /// <param name="height2"></param>
        /// <returns></returns>
        public static bool RectsOverlap(int x1, int y1, int width1, int height1, int x2, int y2, int width2, int height2)
        {
            return Xors3d.RectsOverlap(x1, y1, width1, height1, x2, y2, width2, height2) == 1;
        }

        /// <summary>
        /// Sets viewport form 2D rendering
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        public static void Viewport(int x, int y, int width, int height)
        {
            Xors3d.Viewport(x, y, width, height);
        }

        /// <summary>
        /// Draws an oval shape at the screen coordinates of your choice. 
        /// You can make the oval solid or hollow. 
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="solid"></param>
        public static void DrawOval(int x, int y, int width, int height, bool solid = false)
        {
            Xors3d.Oval(x, y, width, height, solid);
        }

        /// <summary>
        /// Sets a point of origin for all subsequent drawing commands
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public static void SetOrigin(int x, int y)
        {
            Xors3d.Origin(x, y);
        }

        /// <summary>
        /// Returns max supported antialiasing level
        /// </summary>
        /// <returns></returns>
        public static AntialiasTypes GetMaxAntiAlias()
        {
            return (AntialiasTypes)Xors3d.GetMaxAntiAlias();
        }

        /// <summary>
        /// Returns the best supported texture filtering mode
        /// </summary>
        /// <returns></returns>
        public static TextureFilter GetMaxTextureFiltering()
        {
            return (TextureFilter)Xors3d.GetMaxTextureFiltering();
        }

        /// <summary>
        /// Sets anitaliasing type. 
        /// This command must be called before xGaphics3D() 
        /// </summary>
        /// <param name="value"></param>
        public static void SetAntialias(AntialiasTypes value)
        {
            Xors3d.SetAntiAliasType((int)value);
        }

        /// <summary>
        /// Allows you to set the text of the program's title bar
        /// </summary>
        /// <param name="value"></param>
        public static void SetAppTitle(string value)
        {
            Xors3d.AppTitle(value);
        }

        /// <summary>
        /// Sets the window for rendering. 
        /// It may be used by map editor or other tools to render into some gadgets
        /// </summary>
        /// <param name="handle"></param>
        public static void SetRenderWindow(IntPtr handle)
        {
            Xors3d.SetRenderWindow(handle.ToInt32());
        }

        /// <summary>
        /// Sets the top-level window for input devices. 
        /// If the rendering window set by xSetRenderWindow is not a top-level window you should specify a valid top-level window to allow Xors3d manage input devices successfull
        /// </summary>
        /// <param name="handle"></param>
        public static void SetTopWindow(IntPtr handle)
        {
            Xors3d.SetTopWindow(handle.ToInt32());
        }

        /// <summary>
        /// Destroys the window for rendering set by xSetRenderWindow() 
        /// It may be useful if xGraphics3d() is called several times during runtime
        /// </summary>
        public static void DestroyRenderWindow()
        {
            Xors3d.DestroyRenderWindow();
        }

        /// <summary>
        /// Switches the front buffer and back buffer. 
        /// This command should be used when you are using double buffering. Double buffering is a technique used to ensure that screen updates are not visible to the user. If you draw directly to the front buffer, the display may appear flickery as the updates are drawn directly to the screen. If you draw to the back buffer, the updates are drawn in memory. xFlip() is then used to make the back buffer the front buffer, and hence show the updates on screen in one go. At the same time, the front buffer becomes the back buffer, allowing you to draw the next screen update on the back buffer before flipping again. 
        /// </summary>
        public static void Flip()
        {
            Xors3d.Flip();
        }

        /// <summary>
        /// Returns a backbuffer handle. 
        /// This is a value usually used with xSetBuffer() to denote the secondary non-visible drawing buffer called the Back Buffer. In MOST gaming situations, you will want to be using the xBackBuffer() for drawing operations then using xFlip() to bring that buffer to the front buffer where it can be seen. There are other uses for the command, but this is the biggie. See xSetBuffer() for more info. Once again - if you set drawing operations to the xBackBuffer() you will NOT see any of them until you call xFlip(). 
        /// </summary>
        /// <returns></returns>
        public static Buffer GetBackBuffer()
        {
            Buffer buf = new Buffer();
            buf._handle = Xors3d.BackBuffer();
            return buf._handle > 0 ? buf : null;
        }

        /// <summary>
        /// Returns current rendering buffer
        /// </summary>
        /// <returns></returns>
        public static Buffer GetCurrentBuffer()
        {
            Buffer buf = new Buffer();
            buf._handle = Xors3d.GetCurrentBuffer();
            return buf._handle > 0 ? buf : null;
        }

        /// <summary>
        /// Creates and starts stopwatch, returns its handle
        /// </summary>
        /// <returns></returns>
        public static TimeStamp CatchTimeStamp()
        {
            TimeStamp t = new TimeStamp();
            t._handle = Xors3d.CatchTimestamp();
            return t;
        }

        /// <summary>
        /// Sets the current drawing buffer. 
        /// If buffer is not specified, back buffer is used
        /// </summary>
        /// <param name="buffer"></param>
        public static void SetBuffer(Buffer buffer)
        {
            Xors3d.SetBuffer(buffer != null ? buffer._handle : 0);
        }

        /// <summary>
        /// Sets texture as a channel of MRT. 
        /// This function is used for MRT realization, it sets a texture as one of MRT channels
        /// </summary>
        /// <param name="texture"></param>
        /// <param name="frame"></param>
        /// <param name="index"></param>
        public static void SetMrt(Texture texture, int frame, int index)
        {
            Xors3d.SetMRT(texture._handle, frame, index);
        }

        /// <summary>
        /// Unsets all MRT channels. 
        /// This function is used for clearing MRT channel and return rendering to backbuffer
        /// </summary>
        public static void UnSetMrt()
        {
            Xors3d.UnSetMRT();
        }

        /// <summary>
        /// Returns supported MRT number. 
        /// MRT (Multiple Render Targets) refers to the ability to render to multiple textures with a single draw call. These textures can be created independently of each other. It may be used for deferred shading or other effects.
        /// </summary>
        /// <returns></returns>
        public static int GetNumberRT()
        {
            return Xors3d.GetNumberRT();
        }

        /// <summary>
        /// Returns handle of current rendering buffer
        /// </summary>
        /// <returns></returns>
        public static Buffer GetGraphicsBuffer()
        {
            Buffer buf = new Buffer();
            buf._handle = Xors3d.GraphicsBuffer();
            return buf._handle > 0 ? buf : null;
        }

        /// <summary>
        /// This command works like a 'color picker' in paint program
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <returns></returns>
        public static int GetColor(int x, int y)
        {
            return Xors3d.GetColor(x, y);
        }

        /// <summary>
        /// Returns the width of the screen or the virtual viewport. See details. 
        /// Generally the virtual viewport matches the screen. It differs if you use not native aspect ratio. In this case you probably need to know the dimensions of the virtual viewport. See xGraphicsAspectRatio(). 
        /// </summary>
        /// <param name="isVirtual"></param>
        /// <returns></returns>
        public static int GetGraphicsWidth(bool isVirtual = false)
        {
            return Xors3d.GraphicsWidth(isVirtual);
        }

        /// <summary>
        /// Returns the height of the screen or the virtual viewport. See details. 
        /// Generally the virtual viewport matches the screen. It differs if you use not native aspect ratio. In this case you probably need to know the dimensions of the virtual viewport. See xGraphicsAspectRatio(). 
        /// </summary>
        /// <param name="isVirtual"></param>
        /// <returns></returns>
        public static int GetGraphicsHeight(bool isVirtual = false)
        {
            return Xors3d.GraphicsHeight(isVirtual);
        }

        /// <summary>
        /// Returns the color depth of the screen
        /// </summary>
        /// <param name="isVirtual"></param>
        /// <returns></returns>
        public static int GetGraphicsDepth(bool isVirtual = false)
        {
            return Xors3d.GraphicsHeight(isVirtual);
        }

        /// <summary>
        /// Gets or sets the RGB color of the current drawing color
        /// </summary>
        /// <returns></returns>
        public static Color CurrentDrawingColor
        {
            set
            {
                Xors3d.Color(value._red, value._red, value._blue);
            }
            get
            {
                Color c = new Color();
                c._red = Xors3d.ColorRed();
                c._green = Xors3d.ColorGreen();
                c._blue = Xors3d.ColorBlue();
                return c;
            }
        }

        /// <summary>
        /// Set clear color for xCls() command. 
        /// This function changes the color and transparency for subsequent xCls() calls. Use this command when you need xCls() to 'clear' the screen with some other color than black
        /// </summary>
        /// <param name="color"></param>
        public static void SetClsColor(Color color)
        {
            Xors3d.ClsColor(color._red, color._green, color._blue, color._alpha);
        }

        /// <summary>
        /// Clears all entities, brushes and/or textures from the screen and from memory. 
        /// As soon as you clear something, you will not be able to use it again until you reload it. Trying to do so will cause a runtime error. This command is useful for when a level has finished and you wish to load a different level with new entities, brushes and textures.
        /// </summary>
        /// <param name="entities"></param>
        /// <param name="brushes"></param>
        /// <param name="textures"></param>
        public static void ClearWorld(bool entities = true, bool brushes = true, bool textures = true)
        {
            Xors3d.ClearWorld(entities, brushes, textures);
        }

        /// <summary>
        /// Clears rendering buffer. This command will wipe the current drawing buffer clean of any graphics or text present and reset the drawing buffer back to the color defined in the xClsColor() command 
        /// </summary>
        public static void Cls()
        {
            Xors3d.Cls();
        }

        /// <summary>
        /// Renders the specified entity to the current rendering buffer onto the rectangle defined by specified camera xCameraViewport(). 
        /// Xors3D supported directly rendering into images and textures
        /// </summary>
        /// <param name="entity"></param>
        /// <param name="camera"></param>
        /// <param name="tween"></param>
        public static void RenderEntity(Entity entity, Camera camera, float tween = 1)
        {
            Xors3d.RenderEntity(camera._handle, entity._handle, tween);
        }

        /// <summary>
        /// Renders the current scene to the current rendering buffer onto the rectangle defined by each cameras xCameraViewport(). 
        /// Every camera not hidden by xHideEntity() or with a xCameraProjMode() of 0 is rendered. Xors3D supported directly rendering into images and textures. The optional tween parameter should only be specified when RenderWorld is used in conjunction with CaptureWorld. CaptureWorld is used to store the 'old' position, rotation and scale, alpha and colour of each entity in the game world, and a tween value of < 1 will interpolate between these 'old' values and the 'current' ones. A tween value of 0 will render all entities at their state when CaptureWorld was last called, and a tween value of 1 will render all entities at their current state. 
        /// </summary>
        /// <param name="tween"></param>
        /// <param name="renderShadows"></param>
        public static void RenderWorld(float tween = 1.0f, bool renderShadows = false, bool bloom = false)
        {
            Xors3d.RenderWorld(tween, renderShadows);
            if (bloom)
            {
                DrawBloom();
            }
        }

        /// <summary>
        /// Sets global flag which shows if Xors3d should automatically generate tangents and binormals for loaded mesh or animmesh. 
        /// During mesh loading Xors3d generates tangents and binormals automatically by default. In some cases it's not necessary and may be turned off by using a false flag
        /// </summary>
        /// <param name="flag"></param>
        public static void SetAutoTB(bool flag)
        {
            Xors3d.SetAutoTB(flag);
        }

        /// <summary>
        /// Returns max number of suppoted clip planes. 
        /// </summary>
        /// <returns></returns>
        public static int GetMaxClipPlanes()
        {
            return Xors3d.MaxClipPlanes();
        }

        /// <summary>
        /// Enables or disables wireframe rendering. This will show the outline of each polygon on the screen, with no shaded-in areas
        /// </summary>
        /// <param name="state"></param>
        public static void SetWireframe(bool state)
        {
            Xors3d.Wireframe(state ? 1 : 0);
        }

        /// <summary>
        /// Enables or disables hardware dithering. 
        /// Hardware dithering is useful when running games in 16-bit colour mode. Due to the fact that 16-bit mode offers less colours than the human eye can detect, separate bands of colour are often noticeable on shaded objects. However, hardware dithering will dither the entire screen to give the impression that more colours are being used than is actually the case. Due to the fact that 32-bit offer more colours than the human eye can detect, hardware dithering is made pretty much redundant in this mode 
        /// </summary>
        /// <param name="state"></param>
        public static void SetDither(bool state)
        {
            Xors3d.Dither(state ? 1 : 0);
        }

        /// <summary>
        /// Sets skinning type. 
        /// If mesh is loaded by xLoadAnimMesh() it will use skinning method set by this function. You can switch skinning method before loading mesh if you need different skinning types for diffeent meshes 
        /// </summary>
        /// <param name="method"></param>
        public static void SetSkinningMethod(SkinningMethod method)
        {
            Xors3d.SetSkinningMethod((int)method);
        }

        /// <summary>
        /// Returns the number of triangles rendered during last rendering cycle
        /// </summary>
        /// <returns></returns>
        public static int GetTrisRendered()
        {
            return Xors3d.TrisRendered();
        }

        /// <summary>
        /// Returns total number of DIP calls. 
        /// Counter updates during xFlip() call.
        /// </summary>
        /// <returns></returns>
        public static int GetDIP()
        {
            return Xors3d.DIPCounter();
        }

        /// <summary>
        /// Returns total number of rendered surfaces. 
        /// Counter updates during xFlip() call. 
        /// </summary>
        /// <returns></returns>
        public static int GetSurfRendered()
        {
            return Xors3d.SurfRendered();
        }

        /// <summary>
        /// Returns total number of rendered entities. 
        /// Counter updates during xFlip() call. 
        /// </summary>
        /// <returns></returns>
        public static int GetEntityRendered()
        {
            return Xors3d.EntityRendered();
        }

        /// <summary>
        /// Returns FPS (frames per second) counter
        /// </summary>
        /// <returns></returns>
        public static int GetFPS()
        {
            return Xors3d.GetFPS();
        }

        /// <summary>
        /// Enables or disables antialiasing. 
        /// Antialiasing is a technique used to smooth out the entire screen, so that jagged lines are made less noticeable. You must set antialiasing level by xSetAntiAliasType() command before using antialiasing. 
        /// </summary>
        /// <param name="aa"></param>
        public static void SetAntiAliasEnabled(bool state)
        {
            Xors3d.AntiAlias(state ? 1 : 0);
        }

        /// <summary>
        /// Sets global texture filtering method. 
        /// During texture loading the method set by this command will be assigned to this texture. If you use shaders you can change filtering type in shader's code
        /// </summary>
        /// <param name="value"></param>
        public static void SetTextureFiltering(TextureFilter value)
        {
            Xors3d.SetTextureFiltering((int)value);
        }

        /// <summary>
        /// Copys data from back buffer to texture using hardware accelerated method
        /// </summary>
        /// <param name="texture"></param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="filter"></param>
        public static void StretchBackBuffer(Texture texture, int x, int y, int width, int height, TextureFilter filter)
        {
            Xors3d.StretchBackBuffer(texture._handle, x, y, width, height, (int)filter);
        }

        /// <summary>
        /// Returns a pointer to Direct3D device used by Xors3D
        /// </summary>
        /// <returns></returns>
        public static IntPtr GetDevice()
        {
            return (IntPtr)Xors3d.GetDevice();
        }

        /// <summary>
        /// Releases graphics device and clears all resources which are used by it
        /// </summary>
        public static void ReleaseGraphics()
        {
            Xors3d.ReleaseGraphics();
        }

        /// <summary>
        /// Creates a depth-stencil surface. 
        /// Depth-stencil surface is used for z-buffer technique. It's used to write the scene depth for clipping invisible pixels. By default DSS is used with backbuffer dimensions, but if you want to render to larger textures you must create DSS with an appropriate dimensions. E.g. if you use 800x600 mode, default DSS will be 800x600 too. If you want to render to 1024x1024 texture (e.g. for shadows), you must call xCreateDSS(1024, 1024) to make things work. 
        /// </summary>
        /// <param name="width"></param>
        /// <param name="Height"></param>
        public static void CreateDSS(int width, int height)
        {
            Xors3d.CreateDSS(width, height);
        }

        /// <summary>
        /// Deletes a depth-stencil surface
        /// </summary>
        public static void DeleteDSS()
        {
            Xors3d.DeleteDSS();
        }

        /// <summary>
        /// Sets the color for a grid
        /// </summary>
        /// <param name="center">value of the center line </param>
        /// <param name="grid">value of the grid </param>
        public static void SetGridColor(Color center, Color grid)
        {
            Xors3d.GridColor(center._red, center._green, center._blue, grid._red, grid._green, center._blue);
        }

        /// <summary>
        /// Draws a grid in XOZ plane
        /// </summary>
        /// <param name="x"></param>
        /// <param name="z"></param>
        /// <param name="gridSize">Cell size </param>
        /// <param name="range">Number of rows and columns </param>
        public static void DrawGrid(float x, float z, int gridSize, int range)
        {
            Xors3d.DrawGrid(x, z, gridSize, range);
        }

        /// <summary>
        /// Enables or disables rendering of entitys' bounding boxes
        /// </summary>
        /// <param name="draw">True to enable bounding boxes rendering, false to disable </param>
        /// <param name="zOn">True to enable z-buffeing when drawing bounding boxes, false to disable </param>
        /// <param name="color"></param>
        public static void SetDrawBBox(bool draw, bool zOn, Color color)
        {
            Xors3d.DrawBBox(draw, zOn, color._red, color._green, color._blue, color._alpha);
        }

        /// <summary>
        /// Sets 3D Graphics mode. 
        /// This command must be executed before any other 3D command, otherwise programs will return an error. Width and height set the resolution of the screen. The resolution must be compatible with the 3D card and monitor being used. If this value is set to 0, then current system resolution will be used. Depth sets the colour mode of the screen. If this value is omitted or set to 0, then current system colour depth available will be used. Other values usually available are 16 and 32. 16-bit colour mode displays the least amount of colours, 65536. 32-bit colour modes display over 16 million colours and as a result offer a better quality picture, although may result in slower programs than 16-bit. 
        /// Note:
        /// If the size of the window is equal to or greater than the size of desktop, it will be recalculated to fit the screen due to Windows' restrictions. In this case xGraphicsWidth() and xGraphicsHeight() probably won't match the specified width and height. 
        /// </summary>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="depth"></param>
        /// <param name="mode"></param>
        /// <param name="vsync"></param>
        public static void Graphics3D(int width = 1024, int height = 768, int depth = 0, bool mode = false, bool vsync = true)
        {
            Xors3d.Graphics3D(width, height, depth, mode ? 1 : 0, vsync ? 1 : 0);
        }

        /// <summary>
        /// Sets the aspect ratio of the screen
        /// </summary>
        /// <param name="aspect"></param>
        public static void SetAspectRatio(float aspect)
        {
            Xors3d.GraphicsAspectRatio(aspect);
        }

        /// <summary>
        /// Sets the color of the virtual viewport border which is seen if a not native aspect ratio is set. See xGraphicsAspectRatio()
        /// </summary>
        /// <param name="color"></param>
        public static void SetBorderColor(Color color)
        {
            Xors3d.GraphicsBorderColor(color._red, color._green, color._blue);
        }

        /// <summary>
        /// Returns a pointer to Xors3D's rendering window
        /// </summary>
        /// <returns></returns>
        public static IntPtr GetRenderWindow()
        {
            return (IntPtr)Xors3d.GetRenderWindow();
        }

        /// <summary>
        /// Returns true if hardware instancing is supported. 
        /// </summary>
        /// <returns></returns>
        public static bool HWInstancingAvailable()
        {
            return Xors3d.HWInstancingAvailable() == 1;
        }

        /// <summary>
        /// Returns true if shader instancing emulation is supported
        /// </summary>
        /// <returns></returns>
        public static bool ShaderInstancingAvailable()
        {
            return Xors3d.ShaderInstancingAvailable() == 1;
        }

        /// <summary>
        /// Sets or gets a global active shader layer for rendering. 
        /// In Xors3d a mesh or a surface can store up to 8 shaders at the same time. You can switch between them quickly by using this command. 
        /// </summary>
        /// <param name="layer"></param>
        public static int ShaderLayer
        {
            set
            {
                Xors3d.SetShaderLayer(value);
            }
            get
            {
                return Xors3d.GetShaderLayer();
            }
        }

        /// <summary>
        /// Captures the properties (position, rotation, scale, alpha) of each entity in the 3D world. 
        /// This is then used in conjunction with the xRenderWorld() tween parameter in order to render an interpolated frame between the captured state of each entity and the current state of each entity. 
        /// </summary>
        public static void CaptureWorld()
        {
            Xors3d.CaptureWorld();
        }

        /// <summary>
        /// Returns the number of video modes available on the selected graphics card, and sets up the info to be returned by xGfxModeWidth(), xGfxModeHeight(), xGfxModeDepth(). 
        /// </summary>
        /// <returns></returns>
        public static int GetGfxModesCount()
        {
            return Xors3d.CountGfxModes();
        }

        public static int GetGfxModeWidth(int mode)
        {
            return Xors3d.GfxModeWidth(mode);
        }

        public static int GetGfxModeHeight(int mode)
        {
            return Xors3d.GfxModeHeight(mode);
        }

        public static int GetGfxModeDepth(int mode)
        {
            return Xors3d.GfxModeDepth(mode);
        }

        /// <summary>
        /// Returns true if the user's video card can use this graphic mode and false if not
        /// </summary>
        /// <param name="width"></param>
        /// <param name="height"></param>
        /// <param name="depth"></param>
        /// <returns></returns>
        public static bool isGfxModeExists(int width, int height, int depth)
        {
            return Xors3d.GfxModeExists(width, height, depth) == 1;
        }

        /// <summary>
        /// Allows you to enable / disable the frame of the window. Should be called before xGraphics3D(). By default window frame is enabled
        /// </summary>
        /// <param name="value"></param>
        public static void SetWindowFrameEnabled(bool value)
        {
            Xors3d.AppWindowFrame(value?1:0);
        }

        /// <summary>
        /// Returns current time in millicesonds
        /// </summary>
        /// <returns></returns>
        public static int MilliSecs()
        {
            return Xors3d.Millisecs();
        }

        /// <summary>
        /// Returns time in milliseconds elapsed since the engine initialization or last xFlip() call.
        /// </summary>
        /// <param name="fromInit"></param>
        /// <returns></returns>
        public static int DeltaTime(bool fromInit = false)
        {
            return Xors3d.DeltaTime(fromInit);
        }

        /// <summary>
        /// Returns a value interpolated by elapsed time
        /// </summary>
        /// <param name="value"></param>
        /// <param name="time"></param>
        /// <returns></returns>
        public static float GetDeltaValue(float value, int time = 0)
        {
            return Xors3d.DeltaValue(value, time);
        }

    }
}
