﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public static class Joystick
    {
        /// <summary>
        /// Returns joystick type. 
        /// This command returns the type of joystick which is currently connected to the computer. See 'Joystick types' for return values
        /// </summary>
        /// <param name="portID"></param>
        /// <returns></returns>
        public static JoyType GetType(int portID = 0)
        {
            return (JoyType)Xors3d.JoyType(portID);
        }

        /// <summary>
        /// Detects if a joystick button is being held down. 
        /// This command (and its counterparts xKeyDown and xMouseDown) is used to detect if a joystick button is being held down. You must check for each joystick button independantly with its corresponding number
        /// </summary>
        /// <param name="key"></param>
        /// <param name="portID"></param>
        /// <returns></returns>
        public static bool JoyDown(int key, int portID = 0)
        {
            return Xors3d.JoyDown(key, portID) == 1;
        }

        /// <summary>
        /// Returns the number of times a specified button has been hit. 
        /// This command returns the number of times a specified joystick button has been hit since the last time you called the xJoyHit() command 
        /// </summary>
        /// <param name="key"></param>
        /// <param name="portID"></param>
        /// <returns></returns>
        public static bool JoyHit(int key, int portID = 0)
        {
            return Xors3d.JoyHit(key, portID) == 1;
        }

        /// <summary>
        /// Returns pressed joystick button. 
        /// Unlike the other similar commands (xJoyDown() and xJoyHit()), this command doesn't need to know which button you are trying to test for. It looks for any joystick button, then returns the number the user pressed. Since you are polling all the buttons instead of just a specific one, this may be a tad less efficient than using xJoyDown() or xJoyHit(). 
        /// </summary>
        /// <param name="portID"></param>
        /// <returns></returns>
        public static int GetJoy(int portID = 0)
        {
            return Xors3d.GetJoy(portID);
        }

        /// <summary>
        /// Clears information about joystick keys hits. 
        /// There are many times when you aren't interested in the dozens of possible joystick button pressed the player might have made before you are checking for one in particular. Or perhaps you want to pause the game and wait for any joystick button to be hit, but you don't want a 'queued' button press bypassing this. Use this command before you specifically want to poll a joystick button hit from the user. 
        /// </summary>
        public static void Flush()
        {
            Xors3d.FlushJoy();
        }

        /// <summary>
        /// Stops program execution unless any joystick key is not pressed. 
        /// This command makes your program halt until a joystick button is pressed on the joystick. Used alone, it simply halts and waits for a button press. It return pressed button's code value.
        /// </summary>
        /// <param name="portID"></param>
        public static void WaitJoy(int portID = 0)
        {
            Xors3d.WaitJoy(portID);
        }

        public static float GetJoyX(int portID = 0)
        {
            return Xors3d.JoyX(portID);
        }

        public static float GetJoyY(int portID = 0)
        {
            return Xors3d.JoyY(portID);
        }

        public static float GetJoyZ(int portID = 0)
        {
            return Xors3d.JoyZ(portID);
        }

        public static float GetJoyU(int portID = 0)
        {
            return Xors3d.JoyU(portID);
        }

        public static float GetJoyV(int portID = 0)
        {
            return Xors3d.JoyV(portID);
        }

        public static float GetJoyPitch(int portID = 0)
        {
            return Xors3d.JoyPitch(portID);
        }

        public static float GetJoyYaw(int portID = 0)
        {
            return Xors3d.JoyYaw(portID);
        }

        public static float GetJoyRoll(int portID = 0)
        {
            return Xors3d.JoyRoll(portID);
        }

        public static float GetJoyHat(int portID = 0)
        {
            return Xors3d.JoyHat(portID);
        }

        public static float GetJoyXDir(int portID = 0)
        {
            return Xors3d.JoyXDir(portID);
        }

        public static float GetJoyYDir(int portID = 0)
        {
            return Xors3d.JoyYDir(portID);
        }

        public static float GetJoyZDir(int portID = 0)
        {
            return Xors3d.JoyZDir(portID);
        }

        public static float GetJoyUDir(int portID = 0)
        {
            return Xors3d.JoyUDir(portID);
        }

        public static float GetJoyVDir(int portID = 0)
        {
            return Xors3d.JoyVDir(portID);
        }
    }
}
