﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public static class Keyboard
    {

        /// <summary>
        /// Flushes all the currently queued keystrokes. 
        /// This command 'resets' or 'empties out' the queue holding the keyboard inputs 
        /// </summary>
        public static void Flush()
        {
            Xors3d.FlushKeys();
        }

        /// <summary>
        /// Returns the number of times a specified key has been hit since the last time you called the xKeyHit() command
        /// </summary>
        /// <returns></returns>
        public static bool KeyHit(int key)
        {
            return Xors3d.KeyHit(key) == 1;
        }

        /// <summary>
        /// Checks if specified key is pressed. 
        /// This command (similar to its counterparts xMouseDown() and xJoyDown()) is used to detect if a key is being held down. This command returns a 0 if the key is not held down, a 1 if the key is held down
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        public static bool KeyDown(int key)
        {
            return Xors3d.KeyDown(key) == 1;
        }

        /// <summary>
        /// Checks if specified key was released. 
        /// This command returns 1 after each release of the key which was pressed before. 
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        public static bool KeyUp(int key)
        {
            return Xors3d.KeyUp(key) == 1;
        }

        /// <summary>
        /// Stops programm until any key is pressed. 
        /// This command makes your program halt until a key is pressed on the keyboard. Used alone, it simply halts and waits for a key press
        /// </summary>
        public static void WaitKey()
        {
            Xors3d.WaitKey();
        }

        /// <summary>
        /// This command will check to see if a key has been pressed and will return its ASCII value
        /// </summary>
        /// <returns></returns>
        public static int GetKey()
        {
            return Xors3d.GetKey();
        }
    }

    public enum TKeys
    {

    }
}
