﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class Light : Entity
    {
        /// <summary>
        /// Sets the range of a light. 
        /// The range of a light is how far it reaches. Everything outside the range of the light will not be affected by it. The value is very approximate, and should be experimented with for best results. Affect only for spot and point light sources
        /// </summary>
        /// <param name="range"></param>
        public void SetRange(float range)
        {
            Xors3d.LightRange(_handle, range);
        }

        /// <summary>
        /// Sets the color of a light
        /// </summary>
        /// <param name="color"></param>
        public void SetColor(Color color)
        {
            Xors3d.LightColor(_handle, color._red, color._green, color._blue);
        }

        /// <summary>
        /// Sets the cone angle for a spot light. 
        /// The default light cone angles setting is 0, 90. Affect only for spot light sources
        /// </summary>
        /// <param name="inner"></param>
        /// <param name="outer"></param>
        public void SetConeAngles(float inner, float outer)
        {
            Xors3d.LightConeAngles(_handle, inner, outer);
        }

        /// <summary>
        /// Sets epsilon values of light source. 
        /// Every light sources has 2 epsilon values for shadows rendering. First value is a depth bias (it's needed to compensate inaccuracy of floating-point values), second value is to prevent shadowing of angularly triangles. Epsilons delete some noise from shadows, but can lead to incorrect rejection of a part of shadow. Default values are optimal, but you may need to adjust them for best results
        /// </summary>
        /// <param name="epsilon1">First epsilon value (compensates inaccuracy of floating-point values) </param>
        /// <param name="epsilon2">Second epsilon value (prevents shadowing of angularly triangles), actually it is a cosine of maximum angle of shadowed triangle</param>
        public void SetShadowEpsilons(float epsilon1, float epsilon2)
        {
            Xors3d.LightShadowEpsilons(_handle, epsilon1, epsilon2);
        }

        /// <summary>
        /// Enables or disables shadow casting for light. 
        /// You should set casting priority for light source greater than zero to enable shadow casting from it. Priority allows you to change shadows quality in your game, for more information see xShadowPriority()
        /// </summary>
        /// <param name="state">Light casting priority </param>
        public bool EnableShadows
        {
            set
            {
                Xors3d.LightEnableShadows(_handle, value?1:0);
            }
            get
            {
                return Xors3d.LightShadowsEnabled(_handle)==1;
            }
        }

        
    }


    
}
