﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class Line3D
    {
        public static Line3D Create(Vector3 from, Vector3 to, Color color, bool useZBuffer = true)
        {
            Line3D line = new Line3D();
            line._handle = Xors3d.CreateLine3D(from._x, from._y, from._z, to._x, to._y, to._z, color.Red, color.Green, color.Blue, color.Alpha, useZBuffer);
            return line._handle > 0 ? line : null;
        }

        internal int _handle = 0;

        /// <summary>
        /// Deletes all the 3d line's nodes. 
        /// If you need to delete the line use xFreeEntity() 
        /// </summary>
        public void Clear()
        {
            Xors3d.ClearLine3D(_handle);
        }

        /// <summary>
        /// Sets a new origin of the 3d line
        /// </summary>
        /// <param name="origin"></param>
        /// <param name="isGlobal"></param>
        public void SetOrigin(Vector3 origin, bool isGlobal = false)
        {
            Xors3d.Line3DOrigin(_handle, origin._x, origin._y, origin._z, isGlobal);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="isGlobal"></param>
        /// <returns></returns>
        public Vector3 GetOrigin(bool isGlobal = false)
        {
            Vector3 v = new Vector3();
            v._x = Xors3d.Line3DOriginX(_handle, isGlobal);
            v._y = Xors3d.Line3DOriginY(_handle, isGlobal);
            v._z = Xors3d.Line3DOriginZ(_handle, isGlobal);
            return v;
        }

        /// <summary>
        /// Adds a node to the end of the line. 
        ///This allows to build a polygonal chain
        /// </summary>
        /// <param name="vec"></param>
        /// <param name="isGlobal">True if the position should be relative to 0, 0, 0. Default value is False</param>
        public void AddNode(Vector3 vec, bool isGlobal = false)
        {
            Xors3d.Line3DAddNode(_handle, vec._x, vec._y, vec._z, isGlobal);
        }

        /// <summary>
        /// Sets or gets line color
        /// </summary>
        public Color Color
        {
            set
            {
                Xors3d.Line3DColor(_handle, value._red, value._green, value._blue, value._alpha);
            }
            get
            {
                Color c = new Color();
                c._red = Xors3d.Line3DRed(_handle);
                c._green = Xors3d.Line3DGreen(_handle);
                c._blue = Xors3d.Line3DBlue(_handle);
                c._alpha = Xors3d.Line3DAlpha(_handle);
                return c;
            }
        }

        /// <summary>
        /// Changes the use of Z-buffering. 
        /// If the use of Z-buffering is disabled the 3d lines will be seen through the entities ignoring the depth
        /// </summary>
        public bool UseZBuffer
        {
            set
            {
                Xors3d.Line3DUseZBuffer(_handle, value);
            }
            get
            {
                return Xors3d.GetLine3DUseZBuffer(_handle) == 1 ? true : false;
            }
        }

        /// <summary>
        /// Returns the number of the nodes in a 3d line. 
        /// At the beginning a 3d line has 1 node
        /// </summary>
        /// <returns></returns>
        public int NodesCount()
        {
            return Xors3d.Line3DNodesCount(_handle);
        }

        /// <summary>
        /// Returns the XYZ coordinate of a 3d line's node
        /// </summary>
        /// <param name="index"></param>
        /// <param name="isGlobal"></param>
        /// <returns></returns>
        public Vector3 GetNodePosition(int index, bool isGlobal=false)
        {
            Vector3 v = new Vector3();
            v._x = Xors3d.Line3DNodeX(_handle, index, isGlobal);
            v._y = Xors3d.Line3DNodeY(_handle, index, isGlobal);
            v._z = Xors3d.Line3DNodeZ(_handle, index, isGlobal);
            return v;
        }

        /// <summary>
        /// Sets a new position of a node
        /// </summary>
        /// <param name="index"></param>
        /// <param name="vec"></param>
        /// <param name="isGlobal"></param>
        public void SetNodePosition(int index, Vector3 vec, bool isGlobal = false)
        {
            Xors3d.Line3DNodePosition(_handle, index, vec._x, vec._y, vec._z, isGlobal);
        }

        /// <summary>
        /// Deletes a specified 3d line's node
        /// </summary>
        /// <param name="index"></param>
        public void DeleteNode(int index)
        {
            Xors3d.DeleteLine3DNode(_handle, index);
        }
    }
}
