﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public static class Mouse
    {
        internal static bool _visible = true;
        public static bool Visible
        {
            set
            {
                if (value) Xors3d.ShowPointer();
                else Xors3d.HidePointer();
                _visible = value;
            }
            get
            {
                return _visible;
            }
        }

        /// <summary>
        /// Flushes out all the queued up mouse button presses. 
        /// There are many times when you aren't interested in the dozens of possible mouse button pressed the player might have made before you are checking for one in particular. Or perhaps you want to pause the game and wait for any mouse button to be hit, but you don't want a 'queued' button press bypassing this. Use this command before you specifically want to poll a mouse button hit from the user 
        /// </summary>
        public static void Flush()
        {
            Xors3d.FlushMouse();
        }

        /// <summary>
        /// Returns the number of times a specified mouse button has been hit since the last time you called the xMouseHit() command
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        public static bool MouseHit(int key)
        {
            return Xors3d.MouseHit(key) == 1;
        }

        /// <summary>
        /// Detects if a mouse button is being held down. 
        /// You must check for each mouse button independantly with its corresponding number. Also see xMouseHit
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        public static bool MouseDown(int key)
        {
            return Xors3d.MouseDown(key) == 1;
        }

        /// <summary>
        /// Checks if specified mouse button was released. 
        /// This command returns 1 after each release of the mouse button which was pressed before
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        public static bool MouseUp(int key)
        {
            return Xors3d.MouseUp(key) == 1;
        }

        /// <summary>
        /// This command will check to see if any mouse button has been pressed and will return its number. Since you are polling all the mouse buttons instead of just a specific one, this may be a tad less efficient than using xMouseDown or xMouseHit
        /// </summary>
        /// <returns></returns>
        public static int GetMouse()
        {
            return Xors3d.GetMouse();
        }

        public static int GetX()
        {
            return Xors3d.MouseX();
        }

        public static int GetY()
        {
            return Xors3d.MouseY();
        }

        /// <summary>
        /// Returns the current position of the mouse wheel on a suitable mouse. It starts off at zero when your program begins. The value of xMouseZ() increases as you move the wheel away from you and decreases as you move it towards you
        /// </summary>
        /// <returns></returns>
        public static int GetZ()
        {
            return Xors3d.MouseZ();
        }

        public static int GetXSpeed()
        {
            return Xors3d.MouseXSpeed();
        }

        public static int GetYSpeed()
        {
            return Xors3d.MouseYSpeed();
        }
        public static int GetZSpeed()
        {
            return Xors3d.MouseZSpeed();
        }

        /// <summary>
        /// Returns the current mouse speed which is set by an end-user using the mouse control panel application. 
        /// The returned value ranges between 1 (slowest) and 20 (fastest).
        /// You can correct the values returned by xMouseXSpeed() and xMouseYSpeed() multiplying them by xMouseSpeed()/20.0 . 
        /// </summary>
        /// <returns></returns>
        public static int GetMouseSpeed()
        {
            return Xors3d.MouseSpeed();
        }

        /// <summary>
        /// Moves the mouse to a designated location. 
        /// Although the mouse isn't visible on the screen, the mouse location is still being tracked and you can attach a graphic to it. However, there are times when you want to put the pointer to a specific location on the screen. 
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public static void MoveMouse(int x, int y)
        {
            Xors3d.MoveMouse(x, y);
        }

    }
}
