﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class PSystem
    {
        internal int _handle = 0;

        public bool equals(PSystem value)
        {
            return value._handle == _handle;
        }

        /// <summary>
        /// Returns a type of particles used in specified particle system. 
        /// If returned value is true point sprites are used. If returned value is false quads are used
        /// </summary>
        /// <returns></returns>
        public bool isPsystemType()
        {
            return Xors3d.PSystemType(_handle) == 1;
        }

        /// <summary>
        /// Sets or gets the blending mode of a particle system
        /// </summary>
        public Blend Blend
        {
            set
            {
                Xors3d.PSystemSetBlend(_handle, (int)value);
            }
            get
            {
                return (Blend)Xors3d.PSystemGetBlend(_handle);
            }
        }

        /// <summary>
        /// Sets or gets the maximum number of particles in a particle system. 
        /// The number of particles in a particle system will never exceed a given value
        /// </summary>
        public int MaxParticles
        {
            set
            {
                Xors3d.PSystemSetMaxParticles(_handle, value);
            }
            get
            {
                return Xors3d.PSystemGetMaxParticles(_handle);
            }
        }

        /// <summary>
        /// Creates a new particle system and returns its handle. 
        /// Particle system is a special entity which unifies emitters and particles. The number of emitters and particles in a particle system is arbitrary. The optional pointSprites parameter allows you to choose the type of particles: point sprites or traditional quads. Point sprites offer a nice advantage over the normal method of rendering particles. Normally you would create a rectangle using four view-aligned vertices set to the appropriate texture coordinates. With point sprites you can simply use one vertex for every particle! Therefore they require less bandwidth, so you can render more particles using point sprites instead of using the old method, and you don't have to manually adjust the vertices to be aligned with the view vector. However, some features of point sprites (e.g. scaling) are not supported on older video cards available. 
        /// </summary>
        /// <param name="pointSprites"></param>
        /// <returns></returns>
        public static PSystem CreatePSystem(bool pointSprites = false)
        {
            PSystem s = new PSystem();
            s._handle = Xors3d.CreatePSystem(pointSprites);
            return s._handle > 0 ? s : null;
        }

    }
}
