﻿using Xors;

namespace Xors.Physics
{
    public class Contacts
    {
        internal Entity _entity = null;

        /// <summary>
        /// Returns the number of contacts of the entity's body
        /// </summary>
        /// <returns></returns>
        public int CountContacts()
        {
            return Xors3d.EntityCountContacts(_entity._handle);
        }

        /// <summary>
        /// Returns an entity which has a contact with a given entity. 
        /// </summary>
        /// <param name="index">Index of contact in range [0; xCountContacts(entity) - 1]  </param>
        /// <returns></returns>
        public Entity GetContact(int index)
        {
            Entity ent = new Entity();
            ent._handle = Xors3d.EntityGetContact(_entity._handle, index);
            return ent._handle > 0 ? ent : null;
        }

        /// <summary>
        /// Returns the world xyz coordinate of a particular contact. 
        /// </summary>
        /// <param name="index">Index of contact in range [0; xCountContacts(entity) - 1]</param>
        /// <returns></returns>
        public Vector3 GetContactParticular(int index)
        {
            Vector3 v = new Vector3();
            v._x = Xors3d.EntityGetContactX(_entity._handle, index);
            v._y = Xors3d.EntityGetContactY(_entity._handle, index);
            v._z = Xors3d.EntityGetContactZ(_entity._handle, index);
            return v;
        }

        /// <summary>
        /// Returns the xyz component of the nornal of a particular contact
        /// </summary>
        /// <param name="index">Index of contact in range [0; xCountContacts(entity) - 1]</param>
        /// <returns></returns>
        public Vector3 GetContactNormalParticular(int index)
        {
            Vector3 v = new Vector3();
            v._x = Xors3d.EntityGetContactNX(_entity._handle, index);
            v._y = Xors3d.EntityGetContactNY(_entity._handle, index);
            v._z = Xors3d.EntityGetContactNZ(_entity._handle, index);
            return v;
        }

        /// <summary>
        /// Returns the distance of a particular contact. Sometimes you need to ignore the contacts whose distance is less than some threshold. 
        /// </summary>
        /// <param name="index">Index of contact in range [0; xCountContacts(entity) - 1]</param>
        /// <returns></returns>
        public float GetContactDistance(int index)
        {
            return Xors3d.EntityGetContactDistance(_entity._handle, index);
        }

        /// <summary>
        /// Returns a contact impulse of an entity. 
        /// </summary>
        /// <param name="index">Index of contact in range [0; xCountContacts(entity) - 1]</param>
        /// <returns></returns>
        public float GetContactImpulse(int index)
        {
            return Xors3d.EntityGetContactImpulse(_entity._handle, index);
        }
    }
}
