﻿namespace Xors.Physics
{
    public class ForceAndImpulse
    {
        internal Entity _entity = null;

        /// <summary>
        /// Applies a force to the center of mass of the entity's body. 
        /// </summary>
        /// <param name="vec">xyz of force</param>
        /// <param name="isGlobal">True to apply a force in global coordinates. False - in local coordinates. Default value is true.</param>
        public void ApplyCentralForce(Vector3 vec, bool isGlobal = true)
        {
            Xors3d.EntityApplyCentralForce(_entity._handle, vec._x, vec._y, vec._z, isGlobal);
        }

        /// <summary>
        /// Applies a impulse to the center of mass of the entity's body. 
        /// </summary>
        /// <param name="vec">xyz of impulse</param>
        /// <param name="isGlobal">True to apply a impulse in global coordinates. False - in local coordinates. Default value is true.</param>
        public void ApplyCentralImpulse(Vector3 vec, bool isGlobal = true)
        {
            Xors3d.EntityApplyCentralImpulse(_entity._handle, vec._x, vec._y, vec._z, isGlobal);
        }

        /// <summary>
        /// Applies a torque to the entity's body.  
        /// </summary>
        /// <param name="vec">xyz of torque</param>
        /// <param name="isGlobal">True to apply a torque in global coordinates. False - in local coordinates. Default value is true.</param>
        public void ApplyTorque(Vector3 vec, bool isGlobal = true)
        {
            Xors3d.EntityApplyTorque(_entity._handle, vec._x, vec._y, vec._z, isGlobal);
        }

        /// <summary>
        /// Applies a torque impulse to the entity's body
        /// </summary>
        /// <param name="vec">Vector of torque</param>
        /// <param name="isGlobal">True to apply a torque impulse in global coordinates. False - in local coordinates. Default value is true</param>
        public void ApplyTorqueImpulse(Vector3 vec, bool isGlobal = true)
        {
            Xors3d.EntityApplyTorqueImpulse(_entity._handle, vec._x, vec._y, vec._z, isGlobal);
        }

        /// <summary>
        /// Applies a force to the entity's body at a particular point relative to the center of this body
        /// </summary>
        /// <param name="vec">the force vector </param>
        /// <param name="point">the point </param>
        /// <param name="isGlobal">True to apply a force in global coordinates. False - in local coordinates. Default value is true</param>
        /// <param name="globalPoint">True if a particular point is in global coordinates. False - in local coordinates. Default value is true</param>
        public void ApplyForce(Vector3 vec, Vector3 point, bool isGlobal = true, bool globalPoint = true)
        {
            Xors3d.EntityApplyForce(_entity._handle, vec._x, vec._y, vec._z, point._x, point._y, point._z, isGlobal, globalPoint);
        }

        /// <summary>
        /// Applies an impulse to the entity's body at a particular point relative to the center of this body
        /// </summary>
        /// <param name="vec">impulse vector </param>
        /// <param name="point">the point </param>
        /// <param name="isGlobal">True to apply an impulse in global coordinates. False - in local coordinates. Default value is true</param>
        /// <param name="globalPoint">True if a particular point is in global coordinates. False - in local coordinates. Default value is true</param>
        public void ApplyImpulse(Vector3 vec, Vector3 point, bool isGlobal = true, bool globalPoint = true)
        {
            Xors3d.EntityApplyImpulse(_entity._handle, vec._x, vec._y, vec._z, point._x, point._y, point._z, isGlobal, globalPoint);
        }

        /// <summary>
        /// Release all forces
        /// </summary>
        public void ReleaseForces()
        {
            Xors3d.EntityReleaseForces(_entity._handle);
        }
    }
}
