﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class Joint
    {
        internal int _handle = 0;

        

        /// <summary>
        /// Disables or enables collision between two bodies connected with the joint. Initially collisions are enabled
        /// </summary>
        public bool Collisions
        {
            set
            {
                Xors3d.JointDisableCollisions(_handle, value == true ? 0 : 1);
            }
        }

        /// <summary>
        /// Disables or enables the joint. Initially joint is enabled
        /// </summary>
        public bool Enabled
        {
            set
            {
                Xors3d.JointEnable(_handle, value == true ? 1 : 0);
            }
            get
            {
                return Xors3d.JointIsEnabled(_handle)==1?true:false;
            }
        }

        /// <summary>
        /// Retrieves an estimated total applied impulse
        /// </summary>
        /// <returns></returns>
        public float GetImpulse()
        {
            return Xors3d.JointGetImpulse(_handle);
        }

        /// <summary>
        /// Frees a joint
        /// </summary>
        public void Free()
        {
            Xors3d.FreeJoint(_handle);
        }


        /// <summary>
        /// Enables a motor on a specific DOF of D6, D6Spring or hinge joints. The last parameter 'index' is used only for D6 and D6Spring joints. See 'Joint types'.
        /// </summary>
        /// <param name="enabled">Enable or disable the motor </param>
        /// <param name="targetVelocity">The target angular velocity </param>
        /// <param name="maxForce">The maximum force of the motor </param>
        /// <param name="index">DOF index (see 'Joint types')</param>
        public void EnableMotor(bool enabled, float targetVelocity, float maxForce, int index = 0)
        {
            Xors3d.JointEnableMotor(_handle, enabled ? 1 : 0, targetVelocity, maxForce, index);
        }

        /// <summary>
        /// Returns first entity
        /// </summary>
        /// <returns></returns>
        public Entity GetEntityA()
        {
            Entity ent = new Entity();
            ent._handle = Xors3d.JointGetEntityA(_handle);
            return ent._handle>0?ent:null;
        }

        /// <summary>
        /// Returns second entity
        /// </summary>
        /// <returns></returns>
        public Entity GetEntityB()
        {
            Entity ent = new Entity();
            ent._handle = Xors3d.JointGetEntityB(_handle);
            return ent._handle > 0 ? ent : null;
        }
        
    }
}
