﻿namespace Xors.Physics
{
    public class JointHinge : Joint
    {
        /// <summary>
        /// Retrieves the current hinge angle
        /// </summary>
        /// <returns></returns>
        public float GetAngle()
        {
            return Xors3d.JointHingeGetAngle(_handle);
        }

        /// <summary>
        /// Sets the axis of the hinge joint
        /// </summary>
        /// <param name="vec">axis vector </param>
        public void SetAxis(Vector3 vec)
        {
            Xors3d.JointHingeSetAxis(_handle, vec._x, vec._y, vec._z);
        }

        /// <summary>
        /// Sets the limits and other parameters of the hinge joint. Sets the lower and upper limits, the softness, the bias factor and the relaxation factor of the hinge joint
        /// </summary>
        /// <param name="lowerLimit"></param>
        /// <param name="upperLimit"></param>
        /// <param name="softness"></param>
        /// <param name="biasFactor"></param>
        /// <param name="relaxationFactor"></param>
        public void SetLimits(float lowerLimit, float upperLimit, float softness = 0.9f, float biasFactor = 0.3f, float relaxationFactor = 1.0f)
        {
            Xors3d.JointHingeSetLimits(_handle, lowerLimit, upperLimit, softness, biasFactor, relaxationFactor);
        }

        /// <summary>
        /// GetLowerLimit
        /// </summary>
        /// <returns></returns>
        public float GetLowerLimit()
        {
            return Xors3d.JointHingeGetLowerLimit(_handle);
        }

        /// <summary>
        /// GetUpperLimit
        /// </summary>
        /// <returns></returns>
        public float GetUpperLimit()
        {
            return Xors3d.JointHingeGetUpperLimit(_handle);
        }

        /// <summary>
        /// Sets the motor target of the hinge joint. xJointHingeMotorTarget sets target angular velocity ( (targetAngle - currentAngle) / deltaTime ) under the hood
        /// </summary>
        /// <param name="targetAngle">The target angle </param>
        /// <param name="deltaTime">The delta time </param>
        public void SetMotorTarget(float targetAngle, float deltaTime)
        {
            Xors3d.JointHingeSetMotorTarget(_handle, targetAngle, deltaTime);
        }

    }
}
