﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Xors.Physics;

namespace Xors
{
    public class Joints
    {
        internal Entity _entity = null;

        /// <summary>
        /// Creates a 'ball' joint between two bodies and returns its handle. See description JOINT_BALL. One of two bodies may be zero. In this case another body will be jointed to a static world
        /// </summary>
        /// <param name="secondBody">The second entity </param>
        /// <param name="pivot">coordinate of the joint's pivot </param>
        /// <param name="isGlobal">1 (true) to use global pivot coodinates. 0 (false) to use local coordinates. Default is 0 (false)</param>
        /// <returns></returns>
        public JointBall CreateBallJoint(Entity secondBody, Vector3 pivot, bool isGlobal = false)
        {
            JointBall j = new JointBall();
            j._handle = Xors3d.CreateBallJoint(_entity._handle, secondBody._handle, pivot.X, pivot.Y, pivot.Z, isGlobal);
            return j._handle > 0 ? j : null;
        }

        /// <summary>
        /// Creates a 'hinge' joint between two bodies and returns its handle. See description JOINT_HINGE. One of two bodies may be zero. In this case another body will be jointed to a static world.
        /// </summary>
        /// <param name="secondBody">the second entity </param>
        /// <param name="pivot">joint's pivot </param>
        /// <param name="axis">the joint's axis </param>
        /// <param name="isGlobal">1 (true) to use global pivot coodinates. 0 (false) to use local coordinates. Default is 0 (false)</param>
        /// <returns></returns>
        public JointHinge CreateHingeJoint(Entity secondBody, Vector3 pivot, Vector3 axis, bool isGlobal = false)
        {
            JointHinge j = new JointHinge();
            j._handle = Xors3d.CreateHingeJoint(_entity._handle, secondBody._handle, pivot._x, pivot._y, pivot._z, axis._x, axis._y, axis.Z, isGlobal);
            return j._handle > 0 ? j : null;
        }

        /// <summary>
        /// Creates a 'D6' joint between two bodies and returns its handle. See description JOINT_D6. One of two bodies may be zero. In this case another body will be jointed to a static world
        /// </summary>
        /// <param name="secondBody">second entity </param>
        /// <param name="pivot1">first joint's pivot </param>
        /// <param name="pivot2">second joint's pivot </param>
        /// <param name="isGlobal1">1 (true) to use global coodinates of the first pivot. 0 (false) to use local coordinates. Default is 0 (false)</param>
        /// <param name="isGlobal2">1 (true) to use global coodinates of the second pivot. 0 (false) to use local coordinates. Default is 0 (false)</param>
        /// <returns></returns>
        public JointD6 CreateD6Joint(Entity secondBody, Vector3 pivot1, Vector3 pivot2, bool isGlobal1 = false, bool isGlobal2=false)
        {
            JointD6 j = new JointD6();
            j._handle = Xors3d.CreateD6Joint(_entity._handle, secondBody._handle, pivot1._x, pivot1._y, pivot1._z, pivot2._x, pivot2._y, pivot2._z, isGlobal1, isGlobal2);
            return j._handle > 0 ? j : null;
        }

        /// <summary>
        /// Creates a 'D6Spring' joint between two bodies and returns its handle. See description JOINT_D6Spring. One of two bodies may be zero. In this case another body will be jointed to a static world
        /// </summary>
        /// <param name="secondBody">second entity </param>
        /// <param name="pivot1">first joint's pivot </param>
        /// <param name="pivot2">second joint's pivot </param>
        /// <param name="isGlobal1">1 (true) to use global coodinates of the first pivot. 0 (false) to use local coordinates. Default is 0 (false)</param>
        /// <param name="isGlobal2">1 (true) to use global coodinates of the second pivot. 0 (false) to use local coordinates. Default is 0 (false)</param>
        /// <returns></returns>
        public JointD6 CreateD6SpringJoint(Entity secondBody, Vector3 pivot1, Vector3 pivot2, bool isGlobal1 = false, bool isGlobal2 = false)
        {
            JointD6 j = new JointD6();
            j._handle = Xors3d.CreateD6SpringJoint(_entity._handle, secondBody._handle, pivot1._x, pivot1._y, pivot1._z, pivot2._x, pivot2._y, pivot2.Z, isGlobal1, isGlobal2);
            return j._handle > 0 ? j : null;
        }

    }
}
