﻿namespace Xors.Physics
{
    public class PhysicsEntity
    {
        private Entity __entity = null;
        private Contacts _Contacts = new Contacts();
        private Filtering _Filtering = new Filtering();
        private ForceAndImpulse _ForceAndImpulse = new ForceAndImpulse();
        private Joints _Joints = new Joints();
        private PhysicsShapes _Shapes = new PhysicsShapes();
        private Vehicle _Vehicle = new Vehicle();

        public Vehicle Vehicle
        {
            get
            {
                return _Vehicle;
            }
        }

        public PhysicsShapes Shapes
        {
            get
            {
                return _Shapes;
            }
        }

        public Contacts Contacts
        {
            get
            {
                return _Contacts;
            }
        }

        public Joints Joints
        {
            get
            {
                return _Joints;
            }
        }

        public ForceAndImpulse ForceAndImpulse
        {
            get
            {
                return _ForceAndImpulse;
            }
        }

        public Filtering Filtering
        {
            get
            {
                return _Filtering;
            }
        }

        internal Entity _entity
        {
            set
            {
                __entity = value;
                Contacts._entity = __entity;
                Filtering._entity = __entity;
                ForceAndImpulse._entity = __entity;
                Joints._entity = __entity;
                Shapes._entity = __entity;
                Vehicle._entity = __entity;
            }
            get
            {
                return __entity;
            }
        }

        /// <summary>
        /// Returns or sets the XYZ-component of the gravity vector for the given entity
        /// </summary>
        public Vector3 Gravity
        {
            set
            {
                Xors3d.EntitySetGravity(_entity._handle, value._x, value._y, value._z);
            }
            get
            {
                Vector3 vec = new Vector3();
                vec._x = Xors3d.EntityGetGravityX(_entity._handle);
                vec._y = Xors3d.EntityGetGravityY(_entity._handle);
                vec._z = Xors3d.EntityGetGravityZ(_entity._handle);
                return vec;
            }
        }

        /// <summary>
        /// Returns true if entity has a physical body. 
        /// </summary>
        /// <returns></returns>
        public bool HasBody()
        {
            return Xors3d.EntityHasBody(_entity._handle)==1?true:false;
        }

        /// <summary>
        /// Get or set entity kinematic
        /// </summary>
        public bool Kinematic
        {
            set
            {
                Xors3d.EntityMakeKinematic(_entity._handle, value ? 1 : 0);
            }
            get
            {
                return Xors3d.EntityIsKinematic(_entity._handle)==1?true:false;
            }
        }

        //set physics simulation on / off
        public bool Simulation
        {
            set
            {
                Xors3d.EntityDisableSimulation(_entity._handle, value ? 0 : 1);
            }
        }

        /// <summary>
        /// Disables or enables sleeping of an entity
        /// </summary>
        public bool Sleeping
        {
            set
            {
                Xors3d.EntityDisableSleeping(_entity._handle, value ? 0 : 1);
            }
            get
            {
                return Xors3d.EntityIsSleeping(_entity._handle)==1?true:false;
            }
        }

        /// <summary>
        /// Sets the threshold of the linear and angular velocities below which an entity falls asleep. 
        ///Initially linear threshold is 0.8, angular threshold is 1.0. 
        /// </summary>
        /// <param name="linearThreshold"></param>
        /// <param name="angularThreshold"></param>
        public void SetSleepingThresholds(float linearThreshold, float angularThreshold)
        {
            Xors3d.EntitySetSleepingThresholds(_entity._handle, linearThreshold, angularThreshold);
        }

        /// <summary>
        /// Returns a linear velocity threshold of an entity
        /// </summary>
        /// <returns></returns>
        public float GetLinearSleepingThreshold()
        {
            return Xors3d.EntityGetLinearSleepingThreshold(_entity._handle);
        }

        /// <summary>
        /// Returns a angular velocity threshold of an entity
        /// </summary>
        /// <returns></returns>
        public float GetAngularSleepingThreshold()
        {
            return Xors3d.EntityGetAngularSleepingThreshold(_entity._handle);
        }

        /// <summary>
        /// Force an entity to sleep
        /// </summary>
        public void Sleep()
        {
            Xors3d.EntitySleep(_entity._handle);
        }

        /// <summary>
        /// Force an entity to wake up
        /// </summary>
        public void WakeUp()
        {
            Xors3d.EntityWakeUp(_entity._handle);
        }

        /// <summary>
        /// Sets a local position of the entity's body. This function is useful in case the automatic entity-body centering doesn't fit your needs
        /// </summary>
        /// <param name="vec">coordinate of the new position </param>
        public void BodyLocalPosition(Vector3 vec)
        {
            Xors3d.EntityBodyLocalPosition(_entity._handle, vec._x, vec._y, vec._z);
        }

        /// <summary>
        /// Sets a local rotation of the entity's body. This function is useful in case the automatic entity-body centering doesn't fit your needs
        /// </summary>
        /// <param name="vec">coordinate of the new rotation </param>
        public void BodyLocalRotation(Vector3 vec)
        {
            Xors3d.EntityBodyLocalRotation(_entity._handle, vec._x, vec._y, vec._z);
        }

        /// <summary>
        /// Sets a local scale of the entity's body. This function is useful in case the automatic entity-body centering doesn't fit your needs
        /// </summary>
        /// <param name="vec">coordinate of the new scale </param>
        public void BodyLocalScale(Vector3 vec)
        {
            Xors3d.EntityBodyLocalScale(_entity._handle, vec._x, vec._y, vec._z);
        }


        /// <summary>
        /// Sets the linear and angular damping of the entity's body. The values of linear and angular damping are clamped to [0.0; 1.0].Default values of the linear and angular damping are 0. 
        /// </summary>
        /// <param name="linear">The coefficient of the linear damping </param>
        /// <param name="angular">The coefficient of the angular damping </param>
        public void SetDamping(float linear, float angular)
        {
            Xors3d.EntitySetDamping(_entity._handle, linear, angular);
        }

        /// <summary>
        /// Returns the coefficient of the linear damping of the entity's body
        /// </summary>
        /// <returns></returns>
        public float GetLinearDamping()
        {
            return Xors3d.EntityGetLinearDamping(_entity._handle);
        }

        /// <summary>
        /// Returns the coefficient of the angular damping of the entity's body
        /// </summary>
        /// <returns></returns>
        public float GetAngularDamping()
        {
            return Xors3d.EntityGetAngularDamping(_entity._handle);
        }

        /// <summary>
        /// Sets or gets a friction coefficient of the entity's body. Default value of the friction is 0.5. 
        /// </summary>
        public float Friction
        {
            get
            {
                return Xors3d.EntityGetFriction(_entity._handle);
            }
            set
            {
                Xors3d.EntitySetFriction(_entity._handle, value);
            }
        }

        /// <summary>
        /// Sets or gets an anisotropic friction coefficient of the entity's body. Anisotropic frisction has different values for different directions. Default value of the friction is (0.5; 0.5; 0.5). 
        /// </summary>
        public Vector3 AnisotropicFriction
        {
            get
            {
                Vector3 v = new Vector3();
                v._x = Xors3d.EntityGetAnisotropicFrictionX(_entity._handle);
                v._y = Xors3d.EntityGetAnisotropicFrictionY(_entity._handle);
                v._z = Xors3d.EntityGetAnisotropicFrictionZ(_entity._handle);
                return v;
            }
            set
            {
                Xors3d.EntitySetAnisotropicFriction(_entity._handle, value._x, value._y, value._z);
            }
        }

        /// <summary>
        /// Sets or gets the linear factor of the entity's body. This could be useful for locking the motion along one or more world axis. For instance. If you're making a 2d game it would be useful to lock the motion in the XY plane and allow rotation around the Z axis. It could be done in a such way: xEntityLinearFactor(ent, 1.0, 1.0, 0.0); xEntityAngularFactor(ent, 0.0, 0.0, 1.0);Default value is (1.0, 1.0, 1.0). 
        /// </summary>
        public Vector3 LinearFactor
        {
            get
            {
                Vector3 v = new Vector3();
                v._x = Xors3d.EntityGetLinearFactorX(_entity._handle);
                v._y = Xors3d.EntityGetLinearFactorY(_entity._handle);
                v._z = Xors3d.EntityGetLinearFactorZ(_entity._handle);
                return v;
            }
            set
            {
                Xors3d.EntitySetLinearFactor(_entity._handle, value._x, value._y, value._z);
            }
        }

        /// <summary>
        /// Sets or gets the angular factor of the entity's body. This could be useful for locking the rotation around one or more world axis. For instance. If you're making a 2d game it would be useful to lock the motion in the XY plane and allow rotation around the Z axis. It could be done in a such way: xEntityLinearFactor(ent, 1.0, 1.0, 0.0); xEntityAngularFactor(ent, 0.0, 0.0, 1.0);Default value is (1.0, 1.0, 1.0). 
        /// </summary>
        public Vector3 AngularFactor
        {
            get
            {
                Vector3 v = new Vector3();
                v._x = Xors3d.EntityGetAngularFactorX(_entity._handle);
                v._y = Xors3d.EntityGetAngularFactorY(_entity._handle);
                v._z = Xors3d.EntityGetAngularFactorZ(_entity._handle);
                return v;
            }
            set
            {
                Xors3d.EntitySetAngularFactor(_entity._handle, value._x, value._y, value._z);
            }
        }

        /// <summary>
        /// Sets the restitution of the entity's body. 
///. The default value of restitution is 0.0.
///If a restitution of the first object is 1.0 and a restitution of the second one is 0.0 then their resulting restitution is 0.0 (product).
///Normally restitution changes from 0.0 to 1.0. However you may experiment with unnatural values.
///0.0 means inelastic collision (no bounce). 1.0 means elastic collision (no enegry lost).
        /// </summary>
        public float Restitution
        {
            get
            {
                return Xors3d.EntityGetRestitution(_entity._handle);
            }
            set
            {
                Xors3d.EntitySetRestitution(_entity._handle, value);
            }
        }

        /// <summary>
        /// Sets or gets a new mass of the entity's body
        /// </summary>
        public float Mass
        {
            get
            {
                return Xors3d.EntityGetMass(_entity._handle);
            }
            set
            {
                Xors3d.EntitySetMass(_entity._handle, value);
            }
        }

        /// <summary>
        /// Sets the linear velocity of the entity's body.
        /// </summary>
        /// <param name="velocity">velocity vector</param>
        /// <param name="isGlobal">True to set the velocity in global coordinates. False - in local coordinates. Default value is true</param>
        public void SetLinearVelocity(Vector3 velocity, bool isGlobal = true)
        {
            Xors3d.EntitySetLinearVelocity(_entity._handle, velocity._x, velocity._y, velocity._z, isGlobal);
        }

        /// <summary>
        /// Sets the angular velocity of the entity's body.
        /// </summary>
        /// <param name="velocity">velocity vector</param>
        /// <param name="isGlobal">True to set the velocity in global coordinates. False - in local coordinates. Default value is true</param>
        public void SetAngularVelocity(Vector3 velocity, bool isGlobal = true)
        {
            Xors3d.EntitySetAngularVelocity(_entity._handle, velocity._x, velocity._y, velocity._z, isGlobal);
        }

        /// <summary>
        /// Returns the xyz-component of the Angular velocity of the entity's body
        /// </summary>
        /// <param name="isGlobal">True to get a velocity in global coordinates. False - in local coordinates. Default value is true</param>
        /// <returns></returns>
        public Vector3 GetAngularVelocity(bool isGlobal)
        {
            Vector3 v = new Vector3();
            v._x = Xors3d.EntityGetAngularVelocityX(_entity._handle, isGlobal);
            v._y = Xors3d.EntityGetAngularVelocityY(_entity._handle, isGlobal);
            v._z = Xors3d.EntityGetAngularVelocityZ(_entity._handle, isGlobal);
            return v;
        }

        /// <summary>
        /// Returns the xyz-component of the linear velocity of the entity's body
        /// </summary>
        /// <param name="isGlobal">True to get a velocity in global coordinates. False - in local coordinates. Default value is true</param>
        /// <returns></returns>
        public Vector3 GetLinearVelocity(bool isGlobal)
        {
            Vector3 v = new Vector3();
            v._x = Xors3d.EntityGetLinearVelocityX(_entity._handle, isGlobal);
            v._y = Xors3d.EntityGetLinearVelocityY(_entity._handle, isGlobal);
            v._z = Xors3d.EntityGetLinearVelocityZ(_entity._handle, isGlobal);
            return v;
        }
    }
}
