﻿namespace Xors.Physics
{
    public static class PhysicsGlobals
    {
        private static PhysicsRaycasting _Raycasting = new PhysicsRaycasting();

        public static PhysicsRaycasting Raycasting
        {
            get
            {
                return _Raycasting;
            }
        }

        /// <summary>
        /// Determines if collision detection is performed between a pair of groups. 
        ///Parameters:
        ///group0 First group in the range [0; 31].  
        ///group1 Second group in the range [0; 31].  
        /// </summary>
        /// <param name="group0"></param>
        /// <param name="group1"></param>
        public static void GetCollisionFilter(int group0, int group1)
        {
            Xors3d.PhysicsGetCollisionFilter(group0, group1);
        }

        /// <summary>
        /// Specifies if the first group reports contacts with the second group. 
        ///Initially all pairs of groups enabled (all the groups report contacts with each other).
        ///There are 32 contact groups (from 0 to 31).
        ///There is a difference which group is set the first and which one is the second. I.e. xPhysicsSetContactFilter(boxes, balls, 0) has the same effect as xPhysicsSetContactFilter(balls, boxes, 0).
        ///This allows the first group to report contacts with the second group, while the second group doesn't report contacts with the first group. This functionality is useful for making triggers / sensors
        /// </summary>
        /// <param name="group0"></param>
        /// <param name="group1"></param>
        /// <param name="state"></param>
        public static void SetContactFilter(int group0, int group1, bool state)
        {
            Xors3d.PhysicsSetContactFilter(group0, group1, state ? 1 : 0);
        }

        /// <summary>
        /// Determines if the first group reports contacts with the second group. 
        ///Parameters:
        ///group0 First group in the range [0; 31].  
        ///group1 Second group in the range [0; 31].  
        /// </summary>
        /// <param name="group0"></param>
        /// <param name="group1"></param>
        public static bool GetContactFilter(int group0, int group1)
        {
            return Xors3d.PhysicsGetContactFilter(group0, group1) == 1 ? true : false;
        }

        /// <summary>
        /// Specifies if the ray group reports hits with the body group. 
        ///Initially all hits are reported.
        ///There are 32 ray groups and 32 body groups (from 0 to 31). 
        /// </summary>
        /// <param name="rayGroup">Ray group in the range [0; 31]. </param>
        /// <param name="bodyGroup">Body group in the range [0; 31]. </param>
        /// <param name="state">1 (true) to enable hit report. 0 (false) to disable</param>
        public static void SetRaycastFilter(int rayGroup, int bodyGroup, bool state)
        {
            Xors3d.PhysicsSetRaycastFilter(rayGroup, bodyGroup, state ? 1 : 0);
        }

        /// <summary>
        /// Determines if the ray group reports hits with the body group. 
        ///Parameters:
        ///rayGroup Ray group in the range [0; 31].  
        ///bodyGroup Body group in the range [0; 31].  
        /// </summary>
        /// <param name="rayGroup"></param>
        /// <param name="bodyGroup"></param>
        /// <returns></returns>
        public static bool GetRaycastFilter(int rayGroup, int bodyGroup)
        {
            return Xors3d.PhysicsGetRaycastFilter(rayGroup, bodyGroup) == 1 ? true : false;
        }

        public static void DebugRender(bool state)
        {
            Xors3d.PhysicsDebugRender(state ? 1 : 0);
        }

    }
}
