﻿using Xors;

namespace Xors.Physics
{
    public class PhysicsRaycasting
    {

        /// <summary>
        /// Returns the number of hits of the last raycast. The returned value is 0 if there were no hits. If there were some hits the returned value is always 1 for the PXRC_SINGLE
        /// </summary>
        /// <returns></returns>
        public int CountHits()
        {
            return Xors3d.PhysicsCountHits();
        }

        /// <summary>
        /// Cast a ray trough the scene and gathers the detailed information on what was hit. 
///There are two types of raycasting: single and multiple. The first one reports about the closest hit. The second one reports about all hits. See Physics raycasting modes.
///You can ignore hitting of some groups of bodies. See xPhysicsSetRaycastFilter() for more details. 
        /// </summary>
        /// <param name="from">coordinate of the ray's start </param>
        /// <param name="to">coordinate of the ray's end</param>
        /// <param name="rcType">Single or multiple raycast type. See Physics raycasting modes</param>
        /// <param name="rayGroup">Group of the ray for the filtering</param>
        public void RayCast(Vector3 from, Vector3 to, int rcType = 1, int rayGroup = 1)
        {
            Xors3d.PhysicsRayCast(from._x, from._y, from._z, to._x, to._y, to._z, rcType, rayGroup);
        }

        /// <summary>
        /// Returns global XYZ coordinate of the hit point of the last raycast
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        public Vector3 GetHitPoint(int index = 0)
        {
            Vector3 v = new Vector3();
            v._x = Xors3d.PhysicsGetHitPointX(index);
            v._y = Xors3d.PhysicsGetHitPointY(index);
            v._z = Xors3d.PhysicsGetHitPointZ(index);
            return v;
        }

        /// <summary>
        /// Returns the XYZ component of the hit normal of the last raycast
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        public Vector3 GetHitNormals(int index = 0)
        {
            Vector3 v = new Vector3();
            v._x = Xors3d.PhysicsGetHitNormalX(index);
            v._y = Xors3d.PhysicsGetHitNormalY(index);
            v._z = Xors3d.PhysicsGetHitNormalZ(index);
            return v;
        }

        /// <summary>
        /// Returns the distance to the hit point from the ray's start
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        public float GetHitDistance(int index = 0)
        {
            return Xors3d.PhysicsGetHitDistance(index);
        }



    }
}
