﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class ShaderEffectSurface
    {
        internal Surface _surface = null;

        public void SetEffect(Effect effect, int index = -1)
        {
            Xors3d.SetSurfaceEffect(_surface._handle, effect._handle, index);
        }

        /// <summary>
        /// Sets shader constant name for passing bones for skinning computing. 
        /// If you use hadrware skinning, you should pass bone transformation matrices to shader and transform vertices in it. This command specify constant name for each surface in mesh separatly
        /// </summary>
        /// <param name="arrayName"></param>
        /// <param name="layer"></param>
        public void SetBonesArrayName(string arrayName, int layer = -1)
        {
            Xors3d.SurfaceBonesArrayName(_surface._handle, arrayName, layer);
        }

        public void SetEffectInt(string param, int value, int layer = -1)
        {
            Xors3d.SurfaceEffectInt(_surface._handle, param, value, layer);
        }

        public void SetEffectBool(string param, bool value, int layer = -1)
        {
            Xors3d.SurfaceEffectBool(_surface._handle, param, value, layer);
        }

        public void SetEffectFloat(string param, float value, int layer = -1)
        {
            Xors3d.SurfaceEffectFloat(_surface._handle, param, value, layer);
        }

        public void SetEffectVector(string param, Vector3 value, float w = 0, int layer = -1)
        {
            Xors3d.SurfaceEffectVector(_surface._handle, param, value._x, value._y, value._z, w, layer);
        }

      

        public void SetEffectTexture(string param, Texture value, int frame = 0, int layer = -1)
        {
            Xors3d.SurfaceEffectTexture(_surface._handle, param, value._handle, frame, layer);
        }

        public void SetEffectVectorArray(string param, IntPtr value, int count, int layer = -1)
        {
            Xors3d.SurfaceEffectVectorArray(_surface._handle, param, value.ToInt32(), count, layer);
        }

        public void SetEffectMatrixArray(string param, IntPtr value, int count, int layer = -1)
        {
            Xors3d.SurfaceEffectMatrixArray(_surface._handle, param, value.ToInt32(), count, layer);
        }

        public void SetEffectFloatArray(string param, IntPtr value, int count, int layer = -1)
        {
            Xors3d.SurfaceEffectFloatArray(_surface._handle, param, value.ToInt32(), count, layer);
        }

        public void SetEffectIntArray(string param, IntPtr value, int count, int layer = -1)
        {
            Xors3d.SurfaceEffectIntArray(_surface._handle, param, value.ToInt32(), count, layer);
        }

        

        public void SetEffectMatrix(string param, int value, int layer = -1)
        {
            Xors3d.SurfaceEffectMatrix(_surface._handle, param, value, layer);
        }


        public void SetEffectMatrixSemantic(string param, EffectMatrixSemantic value, int layer = -1)
        {
            Xors3d.SurfaceEffectMatrixSemantic(_surface._handle, param, (int)value, layer);
        }

        public void DeleteEffectConstant(string name, int layer = -1)
        {
            Xors3d.DeleteSurfaceConstant(_surface._handle, name, layer);
        }

        public void ClearEffectConstants(int layer = -1)
        {
            Xors3d.ClearSurfaceConstants(_surface._handle, layer);
        }
    }
}
