﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class ShaderEffectsEntity
    {
        internal Entity _entity = null;


        public void SetEffect(Effect effect,int index=-1)
        {
            Xors3d.SetEntityEffect(_entity._handle, effect._handle, index);
        }

        /// <summary>
        /// Sets a name of shader constant to pass bones for skinning computing. 
        /// If you use hardware skinning, you must pass bones transformation matrices in shader and transform vertices in it.
        /// </summary>
        /// <param name="arrayName">Name of shader constant for passing bones matrices </param>
        /// <param name="layer">Index of the effect layer in range [0; 7]. If -1 is passed - active layer will be used </param>
        public void SetBonesArrayName(string arrayName, int layer = -1)
        {
            Xors3d.SetBonesArrayName(_entity._handle, arrayName, layer);
        }

        public void SetEffectInt(string param, int value, int layer = -1)
        {
            Xors3d.SetEffectInt(_entity._handle, param, value, layer);
        }

        public void SetEffectBool(string param, bool value, int layer = -1)
        {
            Xors3d.SetEffectBool(_entity._handle, param, value, layer);
        }

        public void SetEffectFloat(string param, float value, int layer = -1)
        {
            Xors3d.SetEffectFloat(_entity._handle, param, value, layer);
        }

        public void SetEffectVector(string param, Vector3 value, float w = 0, int layer = -1)
        {
            Xors3d.SetEffectVector(_entity._handle, param, value._x, value._y, value._z, w, layer);
        }

        public void SetEffectTexture(string param, Texture value, int  frame = 0,   int  layer = -1,    int  isRecursive = 1  )
        {
            Xors3d.SetEffectTexture(_entity._handle, param, value._handle, frame,layer,isRecursive);
        }

        public void SetEffectVectorArray(string param, IntPtr value, int count, int layer = -1)
        {
            Xors3d.SetEffectVectorArray(_entity._handle, param, value.ToInt32(), count, layer);
        }

        public void SetEffectMatrixArray(string param, IntPtr value, int count, int layer = -1)
        {
            Xors3d.SetEffectMatrixArray(_entity._handle, param, value.ToInt32(), count, layer);
        }

        public void SetEffectFloatArray(string param, IntPtr value, int count, int layer = -1)
        {
            Xors3d.SetEffectFloatArray(_entity._handle, param, value.ToInt32(), count, layer);
        }

        public void SetEffectIntArray(string param, IntPtr value, int count, int layer = -1)
        {
            Xors3d.SetEffectIntArray(_entity._handle, param, value.ToInt32(), count, layer);
        }

        

        public void SetEffectMatrix(string param, int value, int layer = -1)
        {
            Xors3d.SetEffectMatrix(_entity._handle, param, value, layer);
        }

        public void SetEffectTextureOfEntity(string param, int index, int layer = -1)
        {
            Xors3d.SetEffectEntityTexture(_entity._handle, param, index, layer);
        }

        public void SetEffectMatrixSemantic(string param, EffectMatrixSemantic value, int layer = -1)
        {
            Xors3d.SetEffectMatrixSemantic(_entity._handle, param, (int)value, layer);
        }

        public void DeleteEffectConstant(string name, int layer = -1)
        {
            Xors3d.DeleteEffectConstant(_entity._handle, name, layer);
        }

        public void ClearEffectConstants(int layer = -1)
        {
            Xors3d.ClearEffectConstants(_entity._handle, layer);
        }

        public void SetEffectTechnique(string name, int layer = -1)
        {
            Xors3d.SetEffectTechnique(_entity._handle, name, layer);
        }

        /// <summary>
        /// Sets or gets an active shader layer for an entity. 
        /// This value overrides a global shader layer. If -1 is passed - a global value will be used 
        /// </summary>
        /// <param name="layer"></param>
        public int ShaderLayer
        {
            set
            {
                Xors3d.SetEntityShaderLayer(_entity._handle, value);
            }
            get
            {
                return Xors3d.GetEntityShaderLayer(_entity._handle);
            }
        }

        /// <summary>
        /// Setting bump for object
        /// </summary>
        /// <param name="dif"></param>
        /// <param name="norm"></param>
        /// <param name="shaderPath"></param>
        public void SetBump(Camera camera, Texture dif, Texture norm, string shaderPath)
        {
            if (camera == null) return;
            if (dif == null) return;
            if (norm == null) return;
            _entity._bump_camera = camera;
            _entity.Control.SetTexture(dif, 0, 0);
            _entity.Control.SetTexture(norm, 0, 1);
            dif.SetTextureBlend(TextureBlend.TEXBLEND_MULTIPLY);
            norm.SetTextureBlend(TextureBlend.TEXBLEND_DOT3);
            Effect bump = ShaderEffects.LoadFXFile(shaderPath);
            if (bump == null) return;
            _entity.ShaderEffect.SetEffect(bump);
            _entity.ShaderEffect.SetEffectTechnique("Bump");
            XE.bumps.Add(_entity);
            _entity._bump = true;
        }
    }
}
