﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class SoundChannel
    {
        internal int _handle = -1;

        public bool equals(SoundChannel value)
        {
            return value._handle == _handle;
        }

        /// <summary>
        /// Stops channel playing
        /// </summary>
        public void Stop()
        {
            Xors3d.StopChannel(_handle);
        }

        /// <summary>
        /// Pauses channel playing. 
        /// When you are playing a sound channel, there may come a time you wish to pause the sound for whatever reason (like to play another sound effect). 
        /// This command does this - and the channel can be resumed with the xResumeChannel() command. You can use xStopChannel() to actually halt the sound. 
        /// </summary>
        public void Pause()
        {
            Xors3d.PauseChannel(_handle);
        }

        /// <summary>
        /// Continues the playing of a sound sample or music track on the given channel after you have temporarily halted playback on that channel (via xPauseChannel()). 
        /// </summary>
        public void Resume()
        {
            Xors3d.ResumeChannel(_handle);
        }

        /// <summary>
        /// Sets channel frequency. 
        /// You can alter the pitch of a sound channel that is playing (or in use and just paused). Use the frequency of your sound as the 'baseline' for pitch change. So if your sample is at 11025 hertz, increase the pitch to 22050 to make the pitch twice as high, 8000 to make it lower, etc. While similar to xSoundPitch(), this command let's you change the pitch individually of each and every channel in use. 
        /// </summary>
        /// <param name="pitch"></param>
        public void SetPitch(int pitch)
        {
            Xors3d.ChannelPitch(_handle, pitch);
        }

        /// <summary>
        /// Sets channel volume. 
        /// While xSoundVolume() happily changes the volume of the entire program, this command will let you adjust volume rates on a 'per channel' basis. Extremely useful. The volume value is a floating point value between 0.0 and 1.0 (0.0f = silence, 0.5 = half volume, 1.0 = full volume). 
        /// </summary>
        /// <param name="volume">Volume level floating value between 0.0 and 1.0</param>
        public void SetVolume(float volume)
        {
            Xors3d.ChannelVolume(_handle, volume);
        }

        /// <summary>
        /// Returns true if specified channel is being played. 
        /// Often you will need to know if a sound channel has completed playing or not. This command will return 1 if the sound is still playing or 0 if it has stopped. Use this to restart your background music or some other sound that might have stopped unintentionally. 
        /// </summary>
        /// <returns></returns>
        public bool isPlaying()
        {
            return Xors3d.ChannelPlaying(_handle) == 1 ? true : false;
        }
    }
}
