﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public static class Sounds
    {
        /// <summary>
        /// Loads a sound file into memory
        /// </summary>
        /// <param name="path">Name of sound file. Formats supported: raw/wav/mp3/ogg </param>
        /// <returns></returns>
        public static Sound LoadSound(string path)
        {
            Sound s = new Sound();
            s._handle = Xors3d.LoadSound(path);
            return s._handle > 0 ? s : null;
        }

        /// <summary>
        /// Loads a 3D sound file into memory
        /// </summary>
        /// <param name="path">Name of sound file. Formats supported: raw/wav/mp3/ogg </param>
        /// <returns></returns>
        public static Sound Load3DSound(string path)
        {
            Sound s = new Sound();
            s._handle = Xors3d.Load3DSound(path);
            return s._handle > 0 ? s : null;
        }

        /// <summary>
        /// This command will load and play a music file. 
        /// You can't 'preload' the audio like you can a sound sample via the xLoadSound() command. Every time you call the xPlayMusic() command, the file is reloaded and played. This means that if you use the command while some graphics are moving on-screen, you may get a slight pause when the hard drive seeks and grabs the music file. To avoid this, you might want to use the xPlaySound()/xLoopSound() commands instead 
        /// </summary>
        /// <param name="path">Name of music file </param>
        /// <returns></returns>
        public static SoundChannel PlayMusic(string path)
        {
            SoundChannel sc = new SoundChannel();
            sc._handle = Xors3d.PlayMusic(path);
            return sc;
        }

        /// <summary>
        /// Returns current listener
        /// </summary>
        /// <returns></returns>
        public static Entity GetListener()
        {
            Entity ent = new Entity();
            ent._handle = Xors3d.GetListener();
            return ent._handle > 0 ? ent : null;
        }

        /// <summary>
        /// Initializes engine's sound system. Call it if you need to load sounds before setting graphics mode (xGraphics3D() will initialize sound itself) 
        /// </summary>
        public static int Init()
        {
            return Xors3d.InitalizeSound();
        }
    }
}
