﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class Terrain : Entity
    {
        /// <summary>
        /// Enables or disables terrain shading. 
        /// Shaded terrains are a little slower than non-shaded terrains, and in some instances can increase the visibility. However, the option is there to have shaded terrains if you wish to do so
        /// </summary>
        public void SetShading(bool value)
        {
             Xors3d.TerrainShading(_handle, value);          
        }

        /// <summary>
        /// Returns the height of the terrain at terrain grid coordinates x, z. 
        /// The value returned is in the range [0.0; 1.0]. 
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public float GetHeight(int x, int y)
        {
            return Xors3d.TerrainHeight(_handle, x, y);
        }

        /// <summary>
        /// Returns terrain size
        /// </summary>
        public int GetSize()
        {
             return Xors3d.TerrainSize(_handle);         
        }

        /// <summary>
        /// Returns the interpolated xyz coordinate on a terrain
        /// </summary>
        /// <param name="vec">World xyz coordinate </param>
        /// <returns></returns>
        public Vector3 GetInterpolated(Vector3 vec)
        {
            return new Vector3(Xors3d.TerrainX(_handle, vec._x, vec._y, vec._z), Xors3d.TerrainY(_handle, vec._x, vec._y, vec._z), Xors3d.TerrainZ(_handle, vec._x, vec._y, vec._z));
        }

        /// <summary>
        /// Sets the height of a point on a terrain.
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="height">Height of point on terrain. Should be in the range [0.0; 1.0]</param>
        /// <param name="realtime"></param>
        public void Modify(int x, int y, float height, bool realtime = false)
        {
            Xors3d.ModifyTerrain(_handle, x, y, height, realtime);
        }

        /// <summary>
        /// Sets the detail level for a terrain. Terrains detail level (default - 2048)
        /// </summary>
        public void SetDetail(int value)
        {
             Xors3d.TerrainDetail(_handle, value);
        }

        /// <summary>
        /// Enables or disables terrain splatting
        /// </summary>
        public void SetSplatting(bool value)
        {
             Xors3d.TerrainSplatting(_handle, value);
        }

        /// <summary>
        /// Applies the streaming texture for the terrain
        /// </summary>
        /// <param name="texture"></param>
        public void SetTexture(TerrainTexture texture)
        {
            Xors3d.TerrainTexture(_handle, texture._handle);
        }

        /// <summary>
        /// Sets a view zone for the streaming terrain. 
        /// By default value is 1, so engine uses 3x3 zone, if you specify 2 engine will use 5x5 zone and so one
        /// </summary>
        /// <param name="viewZone"></param>
        /// <param name="texturingZone"></param>
        public void SetViewZone(int viewZone, int texturingZone = -1)
        {
            Xors3d.TerrainViewZone(_handle, viewZone, texturingZone);
        }

        /// <summary>
        /// Sets the number of LOD levels for the terrains. 
        /// This command must be called before you create any terrain 
        /// </summary>
        /// <param name="lods">Number of terrain's LODs (5 by default)</param>
        public void Lods(int lods)
        {
            Xors3d.TerrainLODs(lods);
        }

    }
}
