﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public static class TextDrawing
    {
        /// <summary>
        /// Loads a font and returns a font handle
        /// </summary>
        /// <param name="name">Name of font to be loaded, e.g. "arial" or a path to the font file</param>
        /// <param name="height">Height of font in points </param>
        /// <param name="bold">True to load bold version of font, false not to </param>
        /// <param name="italic">True to load italic version of font, false not to </param>
        /// <param name="underline">True to load underlined version of font, false not to</param>
        /// <param name="fontface">Specify an exact name of the fontface if the engine fails to load the font using only the file name</param>
        /// <returns></returns>
        public static Font LoadFont(string name, int height, bool bold = false, bool italic = false, bool underline = false, string fontface = "")
        {
            Font f = new Font();
            f._handle = Xors3d.LoadFont(name, height, bold, italic, underline, fontface);
            return f._handle>0?f:null;
        }

        /// <summary>
        /// Prints a string at the designated screen coordinates
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="textString"></param>
        /// <param name="centerx"></param>
        /// <param name="centery"></param>
        public static void Text(float x, float y, string textString, bool centerx = false, bool centery = false)
        {
            Xors3d.Text(x, y, textString, centerx, centery);
        }

        /// <summary>
        /// Returns the width, in pixels, of the currently selected font. 
        /// Use currently setted font by xSetFont() command
        /// </summary>
        /// <returns></returns>
        public static int GetFontWidth()
        {
            return Xors3d.FontWidth();
        }

        /// <summary>
        /// Returns the height, in pixels, of the currently selected font. 
        /// Use currently setted font by xSetFont() command. 
        /// </summary>
        /// <returns></returns>
        public static int GetFontHeight()
        {
            return Xors3d.FontWidth();
        }

        /// <summary>
        /// Returns the size, in pixels, the width of the indicated string. 
        /// This is useful for determining screen layout, scrolling of text, and more. This is calculated based on the size of the currently loaded font
        /// </summary>
        /// <param name="text"></param>
        /// <returns></returns>
        public static int GetStringWidth(string text)
        {
            return Xors3d.StringWidth(text);
        }

        /// <summary>
        /// Returns the size, in pixels, the height of the indicated string. 
        /// This is useful for determining screen layout, scrolling of text, and more. This is calculated based on the size of the currently loaded font. 
        /// </summary>
        /// <param name="text"></param>
        /// <returns></returns>
        public static int GetStringHeight(string text)
        {
            return Xors3d.StringHeight(text);
        }
    }
}
