﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Xors
{
    public class VideoFile
    {
        internal int _handle = 0;

        public bool equals(VideoFile value)
        {
            return value._handle == _handle;
        }

        /// <summary>
        /// Closes previously opened video file
        /// </summary>
        public void Close()
        {
            Xors3d.CloseMovie(_handle);
        }

        /// <summary>
        /// Draws video file on to the screen
        /// </summary>
        /// <param name="x">Position on the x-axis of top left point of movie </param>
        /// <param name="y">Position on the y-axis of top left point of movie </param>
        /// <param name="width">Width of movie </param>
        /// <param name="height">Height of movie </param>
        public void Draw(int x = 0, int y = 0, int width = -1, int height = -1)
        {
            Xors3d.DrawMovie(_handle, x, y, width, height);
        }


        public int GetWidth()
        {
            return Xors3d.MovieWidth(_handle);
        }


        public int GetHeight()
        {
            return Xors3d.MovieHeight(_handle);
        }

        /// <summary>
        /// Returns true if the specified video file is playing
        /// </summary>
        /// <returns></returns>
        public bool MoviePlaying()
        {
            return  Xors3d.MoviePlaying(_handle) == 1;
        }

        /// <summary>
        /// Sets playback position for the specified video file
        /// </summary>
        /// <param name="time">Playback time in seconds </param>
        /// <param name="relative">If true - playback time will set relative to the current position </param>
        public void Seek(float time, bool relative = false)
        {
            Xors3d.MovieSeek(_handle, time, relative);
        }

        /// <summary>
        /// Returns video file length in seconds
        /// </summary>
        /// <returns></returns>
        public float GetLength()
        {
            return Xors3d.MovieLength(_handle);
        }

        /// <summary>
        /// Returns video file current time
        /// </summary>
        /// <returns></returns>
        public float GetCurrentTime()
        {
            return Xors3d.MovieCurrentTime(_handle);
        }

        /// <summary>
        /// Pause video file playing, or resume
        /// </summary>
        public bool Pause
        {
            set
            {
                if (value) Xors3d.MoviePause(_handle);
                else Xors3d.MovieResume(_handle);
            }
        }

        /// <summary>
        /// Returns texture that represent current video frame
        /// </summary>
        /// <returns></returns>
        public Texture GetTexture()
        {
            Texture tex = new Texture();
            tex._handle = Xors3d.MovieTexture(_handle);
            return tex._handle > 0 ? tex : null;
        }
    }
}
